@@ -2,7 +2,7 @@
 /**
  *  @brief OptimizationResult stores the result of a OptimizationAlgorithmImplementation
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -116,18 +116,6 @@ void OptimizationResult::setCallsNumber(const UnsignedInteger callsNumber)
   callsNumber_ = callsNumber;
 }
 
-UnsignedInteger OptimizationResult::getEvaluationNumber() const
-{
-  LOGWARN("OptimizationResult.getEvaluationNumber is deprecated, use getCallsNumber");
-  return getCallsNumber();
-}
-
-void OptimizationResult::setEvaluationNumber(const UnsignedInteger evaluationNumber)
-{
-  LOGWARN("OptimizationResult.setEvaluationNumber is deprecated, use setCallsNumber");
-  setCallsNumber(evaluationNumber);
-}
-
 /* Iteration number accessor */
 UnsignedInteger OptimizationResult::getIterationNumber() const
 {
@@ -391,41 +379,42 @@ Graph OptimizationResult::drawErrorHistory() const
 {
   if (getProblem().getObjective().getOutputDimension() > 1)
     throw NotYetImplementedException(HERE) << "drawErrorHistory is not available for multi-objective";
-  Graph result("Error history", iterationNumber_ > 0 ? "Iteration number" : "Evaluation number", "Error value", true, "topright", 1.0, GraphImplementation::LOGY);
+  Graph result("Error history", iterationNumber_ > 0 ? "Iteration number" : "Evaluation number", "Error value", true, "topright");
+  result.setLogScale(GraphImplementation::LOGY);
   result.setGrid(true);
   result.setGridColor("black");
-// create a sample with the iteration number to be plotted as x data
+  // create a sample with the iteration number to be plotted as x data
   const UnsignedInteger size = getAbsoluteErrorHistory().getSize();
   {
     Sample data(getAbsoluteErrorHistory());
     for (UnsignedInteger i = 0; i < size; ++i) if (data(i, 0) <= 0.0) data(i, 0) = SpecFunc::ScalarEpsilon;
-    Curve absoluteErrorCurve( data, "absolute error" );
+    Curve absoluteErrorCurve(data, "absolute error");
     absoluteErrorCurve.setLegend("absolute error");
     result.add( absoluteErrorCurve );
   }
-// Relative error
+  // Relative error
   {
     Sample data(getRelativeErrorHistory());
     for (UnsignedInteger i = 0; i < size; ++i) if (data(i, 0) <= 0.0) data(i, 0) = SpecFunc::ScalarEpsilon;
-    Curve relativeErrorCurve( data, "relative error" );
+    Curve relativeErrorCurve(data, "relative error");
     relativeErrorCurve.setLegend("relative error");
     result.add( relativeErrorCurve );
   }
-// Residual error
+  // Residual error
   {
     Sample data(getResidualErrorHistory());
     for (UnsignedInteger i = 0; i < size; ++i) if (data(i, 0) <= 0.0) data(i, 0) = SpecFunc::ScalarEpsilon;
-    Curve residualErrorCurve( data, "residual error" );
+    Curve residualErrorCurve(data, "residual error");
     residualErrorCurve.setLegend("residual error");
     result.add( residualErrorCurve );
   }
-// Constraint error
+  // Constraint error
   {
     Sample data(getConstraintErrorHistory());
     for (UnsignedInteger i = 0; i < size; ++i) if (data(i, 0) <= 0.0) data(i, 0) = SpecFunc::ScalarEpsilon;
-    Curve constraintErrorCurve( data, "constraint error" );
+    Curve constraintErrorCurve(data, "constraint error");
     constraintErrorCurve.setLegend("constraint error");
-    result.add( constraintErrorCurve );
+    result.add(constraintErrorCurve);
   }
   result.setYMargin(0.0);// tighten the Y axis
   return result;
@@ -436,13 +425,15 @@ Graph OptimizationResult::drawOptimalValueHistory() const
 {
   if (getProblem().getObjective().getOutputDimension() > 1)
     throw NotYetImplementedException(HERE) << "drawOptimalValueHistory is not available for multi-objective";
-  Graph result("Optimal value history", iterationNumber_ > 0 ? "Iteration number" : "Evaluation number", "Optimal value", true, "topright", 1.0);
+  if (!getOptimalPoint().getDimension())
+    throw InvalidDimensionException(HERE) << "drawOptimalValueHistory cannot be called without feasible point";
+  Graph result("Optimal value history", iterationNumber_ > 0 ? "Iteration number" : "Evaluation number", "Optimal value", true, "topright");
   result.setGrid(true);
   result.setGridColor("black");
   const Sample dataX(getInputSample());
   Sample dataY(getOutputSample().getMarginal(0));
   const UnsignedInteger size = dataY.getSize();
-  Scalar bestY = getProblem().isMinimization() ? SpecFunc::MaxScalar : -SpecFunc::MaxScalar;
+  Scalar bestY = getProblem().isMinimization() ? SpecFunc::Infinity : -SpecFunc::Infinity;
   UnsignedInteger firstFeasibleIndex = size;
   const Scalar maximumConstraintError = ResourceMap::GetAsScalar("OptimizationAlgorithm-DefaultMaximumConstraintError");
   for (UnsignedInteger i = 0; i < size; ++ i)
