@@ -2,7 +2,7 @@
 /**
  *  @brief Pagmo optimization algorithm
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -20,10 +20,9 @@
  */
 #include "openturns/Pagmo.hxx"
 #include "openturns/PersistentObjectFactory.hxx"
-#include "openturns/SpecFunc.hxx"
 #include "openturns/OTconfig.hxx"
 #include "openturns/DatabaseFunction.hxx"
-#include "openturns/BootstrapExperiment.hxx"
+#include "openturns/RandomGenerator.hxx"
 
 #ifdef OPENTURNS_HAVE_PAGMO
 #include <pagmo/algorithm.hpp>
@@ -45,7 +44,8 @@
 #endif
 #include <pagmo/algorithms/nsga2.hpp>
 #include <pagmo/algorithms/moead.hpp>
-#if (PAGMO_VERSION_MAJOR * 1000 + PAGMO_VERSION_MINOR) >= 2019
+#define PAGMO_VERSION_NR PAGMO_VERSION_MAJOR * 100000 + PAGMO_VERSION_MINOR * 100 + PAGMO_VERSION_PATCH
+#if PAGMO_VERSION_NR >= 201900
 #include <pagmo/algorithms/moead_gen.hpp>
 #endif
 #include <pagmo/algorithms/maco.hpp>
@@ -63,29 +63,38 @@ struct PagmoProblem
 {
   PagmoProblem() {};
 
-  PagmoProblem(const Pagmo * algorithm)
+  PagmoProblem(const Pagmo *algorithm,
+               Sample *evaluationInputHistory,
+               Sample *evaluationOutputHistory)
     : algorithm_(algorithm)
+    , evaluationInputHistory_(evaluationInputHistory)
+    , evaluationOutputHistory_(evaluationOutputHistory)
   {
     if (!algorithm)
-      throw InvalidArgumentException(HERE) << "Null algo";
+      throw InvalidArgumentException(HERE) << "PagmoProblem null algo";
+    if (!evaluationInputHistory || !evaluationOutputHistory)
+      throw InvalidArgumentException(HERE) << "PagmoProblem null history";
 
     // pagmo wants the integer components grouped at the end, so renumbering is in order
-    Indices renum;
-    Indices renum_inv;
+    Indices pagmoOrder;
     const Indices types(algorithm_->getProblem().getVariablesType());
     for (UnsignedInteger i = 0; i < types.getSize(); ++ i)
       if (types[i] == OptimizationProblemImplementation::CONTINUOUS)
-        renum.add(i);
+        pagmoOrder.add(i);
     for (UnsignedInteger i = 0; i < types.getSize(); ++ i)
       if (types[i] != OptimizationProblemImplementation::CONTINUOUS)
-        renum.add(i);
+        pagmoOrder.add(i);
     Indices ordinal(types.getSize());
     ordinal.fill();
-    if (renum != ordinal)
+    if (pagmoOrder != ordinal)
+    {
+      Indices renum(types.getSize());
+      for (UnsignedInteger i = 0; i < types.getSize(); ++ i)
+        renum[pagmoOrder[i]] = i;
       renum_ = renum;
-
-    evaluationInputHistory_ = Sample(0, algorithm->getProblem().getDimension());
-    evaluationOutputHistory_ = Sample(0, algorithm->getProblem().getObjective().getOutputDimension());
+    }
+    *evaluationInputHistory_ = Sample(0, algorithm->getProblem().getDimension());
+    *evaluationOutputHistory_ = Sample(0, algorithm->getProblem().getObjective().getOutputDimension());
   }
 
   Point renumber(const Point & inP) const
@@ -102,29 +111,30 @@ struct PagmoProblem
   pagmo::vector_double fitness(const pagmo::vector_double & inv) const
   {
     const Point inP(renumber(Point(inv.begin(), inv.end())));
-    evaluationInputHistory_.add(inP);
     Point outP(algorithm_->getProblem().getObjective()(inP));
-    evaluationOutputHistory_.add(outP);
+    evaluationInputHistory_->add(inP);
+    evaluationOutputHistory_->add(outP);
     for (UnsignedInteger i = 0; i < outP.getDimension(); ++ i)
       if (!algorithm_->getProblem().isMinimization(i))
         outP[i] *= -1.0;
-    ++ evaluationNumber_;
     if (algorithm_->getProblem().hasEqualityConstraint())
       outP.add(algorithm_->getProblem().getEqualityConstraint()(inP));
     if (algorithm_->getProblem().hasInequalityConstraint())
       outP.add(-1.0 * algorithm_->getProblem().getInequalityConstraint()(inP));// opposite convention for ineq constraints
 
+    std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+    const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0_).count();
+    if (!(algorithm_->getMaximumTimeDuration() <= 0.0) && (timeDuration > algorithm_->getMaximumTimeDuration()))
+      throw TimeoutException(HERE) << "Duration (" << timeDuration << "s) exceeds maximum duration (" << algorithm_->getMaximumTimeDuration() << " s)";
+
     // callbacks
     if (algorithm_->progressCallback_.first)
     {
-      algorithm_->progressCallback_.first((100.0 * evaluationNumber_) / (algorithm_->getStartingSample().getSize() * algorithm_->getMaximumIterationNumber()), algorithm_->progressCallback_.second);
-    }
-    if (algorithm_->stopCallback_.first)
-    {
-      const Bool stop = algorithm_->stopCallback_.first(algorithm_->stopCallback_.second);
-      if (stop)
-        throw InternalException(HERE) << "User stopped optimization";
+      const UnsignedInteger callsNumber = evaluationInputHistory_->getSize();
+      algorithm_->progressCallback_.first((100.0 * callsNumber) / (algorithm_->getStartingSample().getSize() * (algorithm_->getMaximumIterationNumber() + 1)), algorithm_->progressCallback_.second);
     }
+    if (algorithm_->stopCallback_.first && algorithm_->stopCallback_.first(algorithm_->stopCallback_.second))
+      throw InterruptionException(HERE) << "User stopped optimization";
     return outP.toStdVector();
   }
 
@@ -168,6 +178,7 @@ struct PagmoProblem
     const UnsignedInteger blockSize = algorithm_->getBlockSize();
     const UnsignedInteger size = xs.size() / inputDimension;
     const UnsignedInteger blockNumber = static_cast<UnsignedInteger>(ceil(1.0 * size / blockSize));
+
     UnsignedInteger totalDimension = outputDimension;
     if (problem.hasEqualityConstraint())
       totalDimension += problem.getEqualityConstraint().getOutputDimension();
@@ -184,32 +195,34 @@ struct PagmoProblem
         const Point xsi(xs.begin() + offset + i * inputDimension, xs.begin() + offset + (i + 1) * inputDimension);
         inSb[i] = renumber(xsi);
       }
-      evaluationInputHistory_.add(inSb);
       Sample outSb(problem.getObjective()(inSb));
-      evaluationOutputHistory_.add(outSb);
+      evaluationInputHistory_->add(inSb);
+      evaluationOutputHistory_->add(outSb);
       for (UnsignedInteger i = 0; i < effectiveBlockSize; ++ i)
         for (UnsignedInteger j = 0; j < outputDimension; ++ j)
           if (!problem.isMinimization(j))
             outSb(i, j) *= -1.0;
+
       if (problem.hasEqualityConstraint())
         outSb.stack(problem.getEqualityConstraint()(inSb));
       if (problem.hasInequalityConstraint())
         outSb.stack(-1.0 * problem.getInequalityConstraint()(inSb));
       outS.add(outSb);
       offset += blockSize * inputDimension;
-      evaluationNumber_ += effectiveBlockSize;
+
+      std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+      const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0_).count();
+      if (!(algorithm_->getMaximumTimeDuration() <= 0.0) && (timeDuration > algorithm_->getMaximumTimeDuration()))
+        throw TimeoutException(HERE) << "Duration (" << timeDuration << "s) exceeds maximum duration (" << algorithm_->getMaximumTimeDuration() << " s)";
 
       // callbacks
       if (algorithm_->progressCallback_.first)
       {
-        algorithm_->progressCallback_.first((100.0 * evaluationNumber_) / (algorithm_->getStartingSample().getSize() * algorithm_->getMaximumIterationNumber()), algorithm_->progressCallback_.second);
-      }
-      if (algorithm_->stopCallback_.first)
-      {
-        const Bool stop = algorithm_->stopCallback_.first(algorithm_->stopCallback_.second);
-        if (stop)
-          throw InternalException(HERE) << "User stopped optimization";
+        const UnsignedInteger callsNumber = evaluationInputHistory_->getSize();
+        algorithm_->progressCallback_.first((100.0 * callsNumber) / (algorithm_->getStartingSample().getSize() * (algorithm_->getMaximumIterationNumber() + 1)), algorithm_->progressCallback_.second);
       }
+      if (algorithm_->stopCallback_.first && algorithm_->stopCallback_.first(algorithm_->stopCallback_.second))
+        throw InterruptionException(HERE) << "User stopped optimization";
     }
     return outS.getImplementation()->getData().toStdVector();
   }
@@ -235,16 +248,17 @@ struct PagmoProblem
            pagmo::thread_safety::constant : pagmo::thread_safety::none;
   }
 
-  static UnsignedInteger evaluationNumber_;
-  static Sample evaluationInputHistory_;
-  static Sample evaluationOutputHistory_;
-  const Pagmo * algorithm_ = 0;
-  Indices renum_;
-};
+  void setT0(const std::chrono::steady_clock::time_point & t0)
+  {
+    t0_ = t0;
+  }
 
-UnsignedInteger PagmoProblem::evaluationNumber_ = 0;
-Sample PagmoProblem::evaluationInputHistory_;
-Sample PagmoProblem::evaluationOutputHistory_;
+  const Pagmo * algorithm_ = nullptr;
+  Indices renum_;
+  Sample *evaluationInputHistory_ = nullptr;
+  Sample *evaluationOutputHistory_ = nullptr;
+  std::chrono::steady_clock::time_point t0_;
+};
 #endif
 
 
@@ -283,7 +297,7 @@ void Pagmo::checkProblem(const OptimizationProblem & problem) const
   if (problem.hasResidualFunction() || problem.hasLevelFunction())
     throw InvalidArgumentException(HERE) << "Pagmo does not support least squares or nearest point problems";
   const Description multiObjectiveAgorithms = {"nsga2", "moead",
-#if (PAGMO_VERSION_MAJOR * 1000 + PAGMO_VERSION_MINOR) >= 2019
+#if PAGMO_VERSION_NR >= 201900
                                                "moead_gen",
 #endif
                                                "mhaco", "nspso"
@@ -304,13 +318,15 @@ void Pagmo::checkProblem(const OptimizationProblem & problem) const
 
 void Pagmo::run()
 {
-  if (!startingSample_.getSize())
+  Sample startingSample(getStartingSample());
+  UnsignedInteger size = startingSample.getSize();
+  if (!size)
     throw InvalidArgumentException(HERE) << "Starting sample is empty";
-  if (startingSample_.getDimension() != getProblem().getDimension())
+  if (startingSample.getDimension() != getProblem().getDimension())
     throw InvalidArgumentException(HERE) << "Starting sample dimension does not match problem dimension";
-  for (UnsignedInteger i = 0; i < startingSample_.getSize(); ++ i)
+  for (UnsignedInteger i = 0; i < size; ++ i)
   {
-    const Point inP(startingSample_[i]);
+    const Point inP(startingSample[i]);
     if (!getProblem().getBounds().contains(inP))
       LOGWARN(OSS() << "Starting point " << i << " lies outside bounds");
     if (!getProblem().isContinuous())
@@ -327,64 +343,24 @@ void Pagmo::run()
   }
 
 #ifdef OPENTURNS_HAVE_PAGMO
-  PagmoProblem pproblem(this);
+  Sample evaluationInputHistory;
+  Sample evaluationOutputHistory;
+  PagmoProblem pproblem(this, &evaluationInputHistory, &evaluationOutputHistory);
+  std::chrono::steady_clock::time_point t0 = std::chrono::steady_clock::now();
+  pproblem.setT0(t0);
   pagmo::problem prob(pproblem);
   const pagmo::vector_double ctol(prob.get_nc(), getMaximumConstraintError());
   prob.set_c_tol(ctol);
-  const Description constrainedAgorithms = {"gaco", "ihs"};
-  Bool emulatedConstraints = false;
-  if ((getProblem().hasInequalityConstraint() || getProblem().hasEqualityConstraint()) && !constrainedAgorithms.contains(getAlgorithmName()))
+
+  // most algorithms do not support constraints but they can be emulated (by penalization for example)
+  const Bool emulatedConstraints = (getProblem().hasInequalityConstraint() || getProblem().hasEqualityConstraint())
+                                   && !Description({"gaco", "ihs"}).contains(getAlgorithmName());
+  if (emulatedConstraints)
   {
-    emulatedConstraints = true;
     const String unconstrainMethod = ResourceMap::GetAsString("Pagmo-UnconstrainMethod");
     prob = pagmo::unconstrain(prob, unconstrainMethod);
   }
-  pagmo::population pop(prob, 0, 0);
 
-  // nsga2 needs the population size to be a multiple of 4
-  UnsignedInteger populationSize = startingSample_.getSize();
-  if ((algoName_ == "nsga2") && (populationSize % 4))
-  {
-    LOGINFO(OSS() << "Pagmo: must drop the last " << (populationSize % 4) << " points of the initial population for NSGA2 as the size=" << populationSize << " is not a multiple of 4");
-    populationSize = 4 * (populationSize / 4);
-  }
-  // with mhaco starting population must satisfy constraints
-  if ((algoName_ == "mhaco") && (getProblem().hasInequalityConstraint() || getProblem().hasEqualityConstraint()))
-  {
-    Sample startingSampleConstrained(0, getProblem().getDimension());
-    Sample ineqOutput;
-    if (getProblem().hasInequalityConstraint())
-      ineqOutput = getProblem().getInequalityConstraint()(startingSample_);
-    Sample eqOutput;
-    if (getProblem().hasEqualityConstraint())
-      eqOutput = getProblem().getEqualityConstraint()(startingSample_);
-    for (UnsignedInteger i = 0; i < populationSize; ++ i)
-    {
-      Bool ok = true;
-      if (getProblem().hasInequalityConstraint())
-        for (UnsignedInteger j = 0; j < ineqOutput.getDimension(); ++ j)
-          ok = ok && (ineqOutput(i, j) >= -getMaximumConstraintError());
-      if (getProblem().hasEqualityConstraint())
-        for (UnsignedInteger j = 0; j < eqOutput.getDimension(); ++ j)
-          ok = ok && (std::abs(eqOutput(i, j)) <= getMaximumConstraintError());
-      if (ok)
-        startingSampleConstrained.add(startingSample_[i]);
-    }
-    if (!startingSampleConstrained.getSize())
-      throw InvalidArgumentException(HERE) << "No point in starting population satisfies constraints";
-    if (startingSampleConstrained.getSize() < populationSize)
-    {
-      const Indices indices(BootstrapExperiment::GenerateSelection(populationSize, startingSampleConstrained.getSize()));
-      startingSample_ = startingSampleConstrained.select(indices);
-      LOGINFO(OSS() << "Pagmo: Initial population bootstrapped to satisfy constraints");
-    }
-  }
-  for (UnsignedInteger i = 0; i < populationSize; ++ i)
-  {
-    const Point inP(startingSample_[i]);
-    pagmo::vector_double x(pproblem.renumber(inP).toStdVector());
-    pop.push_back(x);
-  }
   pagmo::algorithm algo;
   if (algoName_ == "gaco")
   {
@@ -399,10 +375,15 @@ void Pagmo::run()
     const Scalar focus = ResourceMap::GetAsScalar("Pagmo-gaco-focus");
     const Bool memory = ResourceMap::GetAsBool("Pagmo-memory");
     if (!memory)
-      ker = std::min(ker, populationSize);
-    pagmo::gaco algorithm_impl(getMaximumIterationNumber(), ker, q, oracle, acc, threshold, n_gen_mark, impstop, getMaximumEvaluationNumber(), focus, memory);
+      ker = std::min(ker, size);
+    pagmo::gaco algorithm_impl(getMaximumIterationNumber(), ker, q, oracle, acc, threshold, n_gen_mark, impstop, getMaximumCallsNumber(), focus, memory);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#else
     if (!emulatedConstraints)
       algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
     algo = algorithm_impl;
   }
   else if (algoName_ == "de")
@@ -468,8 +449,13 @@ void Pagmo::run()
     const UnsignedInteger neighb_param = ResourceMap::GetAsUnsignedInteger("Pagmo-pso-neighb_param");
     const Bool memory = ResourceMap::GetAsBool("Pagmo-memory");
     pagmo::pso_gen algorithm_impl(getMaximumIterationNumber(), omega, eta1, eta2, max_vel, variant, neighb_type, neighb_param, memory);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#else
     if (!emulatedConstraints)
       algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
     algo = algorithm_impl;
   }
   else if (algoName_ == "sea")
@@ -517,7 +503,16 @@ void Pagmo::run()
     const Scalar cmu = ResourceMap::GetAsScalar("Pagmo-cmaes-cmu");
     const Scalar sigma0 = ResourceMap::GetAsScalar("Pagmo-cmaes-sigma0");
     const Bool memory = ResourceMap::GetAsBool("Pagmo-memory");
-    algo = pagmo::cmaes(getMaximumIterationNumber(), cc, cs, c1, cmu, sigma0, getMaximumResidualError(), getMaximumAbsoluteError(), memory, getProblem().hasBounds());
+    const Bool force_bounds = getProblem().hasBounds();
+    pagmo::cmaes algorithm_impl(getMaximumIterationNumber(), cc, cs, c1, cmu, sigma0, getMaximumResidualError(), getMaximumAbsoluteError(), memory, force_bounds);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#elif PAGMO_VERSION_NR >= 201900
+    if (!emulatedConstraints)
+      algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
+    algo = algorithm_impl;
   }
   else if (algoName_ == "xnes")
   {
@@ -527,19 +522,33 @@ void Pagmo::run()
     const Scalar eta_b = ResourceMap::GetAsScalar("Pagmo-xnes-eta_b");
     const Scalar sigma0 = ResourceMap::GetAsScalar("Pagmo-xnes-sigma0");
     const Bool memory = ResourceMap::GetAsBool("Pagmo-memory");
-    algo = pagmo::xnes(getMaximumIterationNumber(), eta_mu, eta_sigma, eta_b, sigma0, getMaximumResidualError(), getMaximumAbsoluteError(), memory, getProblem().hasBounds());
+    const Bool force_bounds = getProblem().hasBounds();
+    algo = pagmo::xnes(getMaximumIterationNumber(), eta_mu, eta_sigma, eta_b, sigma0, getMaximumResidualError(), getMaximumAbsoluteError(), memory, force_bounds);
   }
 #endif
   else if (algoName_ == "nsga2")
   {
+    const UnsignedInteger reminder = size % 4;
+    if (reminder)
+    {
+      LOGINFO(OSS() << "Pagmo: must drop the last " << reminder << " points of the initial population for NSGA2 as the size (" << size << ") is not a multiple of 4");
+      size -= reminder;
+      startingSample.split(size);
+    }
+
     // nsga2(unsigned gen = 1u, double cr = 0.95, double eta_c = 10., double m = 0.01, double eta_m = 50., unsigned seed = pagmo::random_device::next())
     const Scalar cr = ResourceMap::GetAsScalar("Pagmo-nsga2-cr");
     const Scalar eta_c = ResourceMap::GetAsScalar("Pagmo-nsga2-eta_c");
     const Scalar m = ResourceMap::GetAsScalar("Pagmo-nsga2-m");
     const Scalar eta_m = ResourceMap::GetAsScalar("Pagmo-nsga2-eta_m");
     pagmo::nsga2 algorithm_impl(getMaximumIterationNumber(), cr, eta_c, m, eta_m);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#else
     if (!emulatedConstraints)
       algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
     algo = algorithm_impl;
   }
   else if (algoName_ == "moead")
@@ -556,7 +565,7 @@ void Pagmo::run()
     const Bool preserve_diversity = ResourceMap::GetAsBool("Pagmo-moead-preserve_diversity");
     algo = pagmo::moead(getMaximumIterationNumber(), weight_generation, decomposition, neighbours, CR, F, eta_m, realb, limit, preserve_diversity);
   }
-#if (PAGMO_VERSION_MAJOR * 1000 + PAGMO_VERSION_MINOR) >= 2019
+#if PAGMO_VERSION_NR >= 201900
   else if (algoName_ == "moead_gen")
   {
     // moead_gen(unsigned gen = 1u, std::string weight_generation = "grid", std::string decomposition = "tchebycheff", population::size_type neighbours = 20u, double CR = 1.0, double F = 0.5, double eta_m = 20., double realb = 0.9, unsigned limit = 2u, bool preserve_diversity = true, unsigned seed = pagmo::random_device::next())
@@ -569,11 +578,52 @@ void Pagmo::run()
     const Scalar realb = ResourceMap::GetAsScalar("Pagmo-moead-realb");
     const UnsignedInteger limit = ResourceMap::GetAsUnsignedInteger("Pagmo-moead-limit");
     const Bool preserve_diversity = ResourceMap::GetAsBool("Pagmo-moead-preserve_diversity");
-    algo = pagmo::moead_gen(getMaximumIterationNumber(), weight_generation, decomposition, neighbours, CR, F, eta_m, realb, limit, preserve_diversity);
+    pagmo::moead_gen algorithm_impl(getMaximumIterationNumber(), weight_generation, decomposition, neighbours, CR, F, eta_m, realb, limit, preserve_diversity);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#else
+    if (!emulatedConstraints)
+      algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
+    algo = algorithm_impl;
   }
 #endif
   else if (algoName_ == "mhaco")
   {
+    // starting population must satisfy constraints
+    if (getProblem().hasInequalityConstraint() || getProblem().hasEqualityConstraint())
+    {
+      Sample startingSampleConstrained(0, getProblem().getDimension());
+      Sample ineqOutput;
+      if (getProblem().hasInequalityConstraint())
+        ineqOutput = getProblem().getInequalityConstraint()(startingSample);
+      Sample eqOutput;
+      if (getProblem().hasEqualityConstraint())
+        eqOutput = getProblem().getEqualityConstraint()(startingSample);
+      for (UnsignedInteger i = 0; i < size; ++ i)
+      {
+        Bool ok = true;
+        if (getProblem().hasInequalityConstraint())
+          for (UnsignedInteger j = 0; j < ineqOutput.getDimension(); ++ j)
+            ok = ok && (ineqOutput(i, j) >= -getMaximumConstraintError());
+        if (getProblem().hasEqualityConstraint())
+          for (UnsignedInteger j = 0; j < eqOutput.getDimension(); ++ j)
+            ok = ok && (std::abs(eqOutput(i, j)) <= getMaximumConstraintError());
+        if (ok)
+          startingSampleConstrained.add(startingSample[i]);
+      }
+      if (!startingSampleConstrained.getSize())
+        throw InvalidArgumentException(HERE) << "No point in starting population satisfies constraints";
+      if (startingSampleConstrained.getSize() < size)
+      {
+        const RandomGenerator::UnsignedIntegerCollection selection(RandomGenerator::IntegerGenerate(size, startingSampleConstrained.getSize()));
+        const Indices indices(selection.begin(), selection.end());
+        startingSample = startingSampleConstrained.select(indices);
+        LOGINFO(OSS() << "Pagmo: Initial population bootstrapped to satisfy constraints");
+      }
+    }
+
     // maco(unsigned gen = 100u, unsigned ker = 63u, double q = 1.0, unsigned threshold = 1u, unsigned n_gen_mark = 7u, unsigned evalstop = 100000u, double focus = 0., bool memory = false, unsigned seed = pagmo::random_device::next())
     UnsignedInteger ker = ResourceMap::GetAsUnsignedInteger("Pagmo-mhaco-ker");
     const Scalar q = ResourceMap::GetAsScalar("Pagmo-mhaco-q");
@@ -582,10 +632,15 @@ void Pagmo::run()
     const Scalar focus = ResourceMap::GetAsScalar("Pagmo-mhaco-focus");
     const Bool memory = ResourceMap::GetAsBool("Pagmo-memory");
     if (!memory)
-      ker = std::min(ker, populationSize);
-    pagmo::maco algorithm_impl(getMaximumIterationNumber(), ker, q, threshold, n_gen_mark, getMaximumEvaluationNumber(), focus, memory);
+      ker = std::min(ker, size);
+    pagmo::maco algorithm_impl(getMaximumIterationNumber(), ker, q, threshold, n_gen_mark, getMaximumCallsNumber(), focus, memory);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#else
     if (!emulatedConstraints)
       algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
     algo = algorithm_impl;
   }
   else if (algoName_ == "nspso")
@@ -600,19 +655,66 @@ void Pagmo::run()
     const String diversity_mechanism = ResourceMap::GetAsString("Pagmo-nspso-diversity_mechanism");
     const Bool memory = ResourceMap::GetAsBool("Pagmo-memory");
     pagmo::nspso algorithm_impl(getMaximumIterationNumber(), omega, c1, c2, chi, v_coeff, leader_selection_range, diversity_mechanism, memory);
+#if PAGMO_VERSION_NR >= 201901
+    // requires https://github.com/esa/pagmo2/pull/575
+    algorithm_impl.set_bfe(pagmo::bfe{});
+#else
     if (!emulatedConstraints)
       algorithm_impl.set_bfe(pagmo::bfe{});
+#endif
     algo = algorithm_impl;
   }
   else
     throw NotYetImplementedException(HERE) << algoName_;
   algo.set_verbosity(Log::HasDebug());
   algo.set_seed(seed_);
-  PagmoProblem::evaluationNumber_ = 0;
+
+  // evaluate initial population
+  pagmo::population pop(prob);
+  // requires https://github.com/esa/pagmo2/pull/575
+#if PAGMO_VERSION_NR >= 201901
+  const OptimizationProblem problem(getProblem());
+  const UnsignedInteger inputDimension = problem.getObjective().getInputDimension();
+  const UnsignedInteger blockSize = getBlockSize();
+  const UnsignedInteger blockNumber = static_cast<UnsignedInteger>(ceil(1.0 * size / blockSize));
+  for (UnsignedInteger outerSampling = 0; outerSampling < blockNumber; ++ outerSampling)
+  {
+    const UnsignedInteger effectiveBlockSize = ((outerSampling == (blockNumber - 1)) && (size % blockSize)) ? (size % blockSize) : blockSize;
+    Sample inSb(effectiveBlockSize, inputDimension);
+    for (UnsignedInteger i = 0; i < effectiveBlockSize; ++ i)
+      inSb[i] = startingSample[i + outerSampling * blockSize];
+    const pagmo::vector_double inV(inSb.getImplementation()->getData().toStdVector());
+    const pagmo::vector_double outV(prob.batch_fitness(inV));
+    const UnsignedInteger nf = outV.size() / effectiveBlockSize;
+    for (UnsignedInteger i = 0; i < effectiveBlockSize; ++ i)
+    {
+      const pagmo::vector_double inVi(inV.begin() + i * inputDimension, inV.begin() + (i + 1) * inputDimension);
+      const pagmo::vector_double outVi(outV.begin() + i * nf, outV.begin() + (i + 1) * nf);
+      pop.push_back(inVi, outVi);
+    }
+  }
+#else
+  for (UnsignedInteger i = 0; i < size; ++ i)
+  {
+    const Point inP(startingSample[i]);
+    pagmo::vector_double x(pproblem.renumber(inP).toStdVector());
+    pagmo::vector_double y(prob.fitness(x));
+    pop.push_back(x, y);
+  }
+#endif
+
+  // evolve initial population over several generations
   pop = algo.evolve(pop);
+
+  // retrieve results
   result_ = OptimizationResult(getProblem());
-  result_.setEvaluationNumber(PagmoProblem::evaluationNumber_);
+  result_.setCallsNumber(evaluationInputHistory.getSize());
   result_.setIterationNumber(getMaximumIterationNumber());
+
+  std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+  const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0).count();
+  result_.setTimeDuration(timeDuration);
+
   Scalar optimalValue = 0.0;
   Sample finalPoints(0, getProblem().getDimension());
 
@@ -651,8 +753,7 @@ void Pagmo::run()
   }
 
   // we want to retrieve evaluations before penalization to avoid MaxScalar values
-  const DatabaseFunction xToY(PagmoProblem::evaluationInputHistory_,
-                              PagmoProblem::evaluationOutputHistory_);
+  const DatabaseFunction xToY(evaluationInputHistory, evaluationOutputHistory);
   const Sample finalValues(xToY(finalPoints));
   result_.setFinalPoints(finalPoints);
   result_.setFinalValues(finalValues);
@@ -685,7 +786,13 @@ void Pagmo::run()
     // retrieve non-penalized output values instead of using pop.get_f
     std::vector<std::vector<double> > popf;
     for (UnsignedInteger i = 0; i < finalValues.getSize(); ++ i)
-      popf.push_back(Point(finalValues[i]).toStdVector());
+    {
+      Point outP(finalValues[i]);
+      for (UnsignedInteger j = 0; j < outP.getDimension(); ++ j)
+        if (!getProblem().isMinimization(j))
+          outP[j] *= -1.0;
+      popf.push_back(outP.toStdVector());
+    }
     // compute the fronts
     std::vector<std::vector<pagmo::pop_size_t> > fronts(std::get<0>(pagmo::fast_non_dominated_sorting(popf)));
     Collection<Indices> frontIndices(fronts.size());
@@ -732,7 +839,6 @@ Point Pagmo::getStartingPoint() const
 /* Starting sample accessor */
 void Pagmo::setStartingSample(const Sample & startingSample)
 {
-//   checkStartingSampleConsistentWithOptimizationProblem(startingSample, getProblem());
   startingSample_ = startingSample;
 }
 
@@ -750,7 +856,7 @@ Description Pagmo::GetAlgorithmNames()
                                  "cmaes", "xnes",
 #endif
                                  "nsga2", "moead",
-#if (PAGMO_VERSION_MAJOR * 1000 + PAGMO_VERSION_MINOR) >= 2019
+#if PAGMO_VERSION_NR >= 201900
                                  "moead_gen",
 #endif
                                  "mhaco", "nspso"
@@ -772,19 +878,6 @@ String Pagmo::getAlgorithmName() const
   return algoName_;
 }
 
-/* Number of generations to evolve */
-void Pagmo::setGenerationNumber(const UnsignedInteger generationNumber)
-{
-  LOGWARN(OSS() << "Pagmo.setGenerationNumber is deprecated in favor of setMaximumIterationNumber");
-  setMaximumIterationNumber(generationNumber);
-}
-
-UnsignedInteger Pagmo::getGenerationNumber() const
-{
-  LOGWARN(OSS() << "Pagmo.getGenerationNumber is deprecated in favor of getMaximumIterationNumber");
-  return getMaximumIterationNumber();
-}
-
 /* Random generator seed accessor */
 void Pagmo::setSeed(const UnsignedInteger seed)
 {
