@@ -2,7 +2,7 @@
 /**
  *  @brief MatrixImplementation implements the classical mathematical MatrixImplementation
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +29,6 @@
 #include "openturns/Lapack.hxx"
 #include "openturns/ResourceMap.hxx"
 #include "openturns/SpecFunc.hxx"
-#include "openturns/Os.hxx"
 #include "openturns/Exception.hxx"
 #include "openturns/Sample.hxx"
 
@@ -1110,47 +1109,6 @@ Scalar MatrixImplementation::computeLogAbsoluteDeterminantSymInPlace(Scalar & si
 {
   symmetrize();
   return computeLogAbsoluteDeterminantInPlace(sign);
-  /* The implementation based on dsytrf does not uses the 2x2 diagonal blocks correctly
-     int n = nbRows_;
-     if (n == 0) throw InvalidDimensionException(HERE);
-     std::vector<int> ipiv (n);
-     char uplo = 'L';
-     int info;
-     Scalar determinant = 1.0;
-     int lwork = -1;
-     double lwork_d;
-     int luplo = 1;
-
-     // LU Factorization with LAPACK
-     if (keepIntact)
-     {
-     MatrixImplementation A(*this);
-     dsytrf_(&uplo, &n, &A[0], &n, &ipiv[0],&lwork_d, &lwork, &info, &luplo);
-     lwork = static_cast<int>(lwork_d);
-     Point work(lwork);
-     dsytrf_(&uplo, &n, &A[0], &n, &ipiv[0],&work[0], &lwork, &info, &luplo);
-     // Determinant computation
-     for (UnsignedInteger i = 0; i < static_cast<UnsignedInteger>(n); ++i)
-     {
-     determinant *= A[i * (ipiv.size() + 1)];
-     if (ipiv[i] != int(i + 1)) determinant = -determinant;
-     }
-     }
-     else
-     {
-     dsytrf_(&uplo, &n, &(*this)[0], &n, &ipiv[0],&lwork_d, &lwork, &info, &luplo);
-     lwork = static_cast<int>(lwork_d);
-     Point work(lwork);
-     dsytrf_(&uplo, &n, &(*this)[0], &n, &ipiv[0],&work[0], &lwork, &info, &luplo);
-     // Determinant computation
-     for (UnsignedInteger i = 0; i < static_cast<UnsignedInteger>(n); ++i)
-     {
-     determinant *= (*this)[i * (ipiv.size() + 1)];
-     if (ipiv[i] != int(i + 1)) determinant = -determinant;
-     }
-     }
-
-     return determinant; */
 }
 
 /* Compute determinant for a symmetric matrix */
