@@ -2,7 +2,7 @@
 /**
  *  @brief The CompositeDistribution distribution
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +29,7 @@
 #include "openturns/MethodBoundEvaluation.hxx"
 #include "openturns/Brent.hxx"
 #include "openturns/SobolSequence.hxx"
-#include "openturns/SymbolicFunction.hxx"
+#include "openturns/IdentityFunction.hxx"
 
 BEGIN_NAMESPACE_OPENTURNS
 
@@ -40,7 +40,7 @@ static const Factory<CompositeDistribution> Factory_CompositeDistribution;
 /* Default constructor */
 CompositeDistribution::CompositeDistribution()
   : DistributionImplementation()
-  , function_(SymbolicFunction("x", "x"))
+  , function_(IdentityFunction(1))
   , antecedent_(Uniform(0.0, 1.0))
   , bounds_(0)
   , values_(0)
@@ -48,7 +48,6 @@ CompositeDistribution::CompositeDistribution()
   , increasing_(0)
   , solver_(Brent(ResourceMap::GetAsScalar("CompositeDistribution-SolverEpsilon"), ResourceMap::GetAsScalar("CompositeDistribution-SolverEpsilon"), ResourceMap::GetAsScalar("CompositeDistribution-SolverEpsilon")))
 {
-  // We don't know if the function is thread-safe and it could be called in parallel in computePDF()
   setParallel(false);
   setName("CompositeDistribution");
   setDimension(1);
@@ -68,8 +67,6 @@ CompositeDistribution::CompositeDistribution(const Function & function,
   , increasing_(0)
   , solver_(Brent(ResourceMap::GetAsScalar("CompositeDistribution-SolverEpsilon") * (antecedent.getRange().getUpperBound()[0] - antecedent.getRange().getLowerBound()[0]), ResourceMap::GetAsScalar("CompositeDistribution-SolverEpsilon"), ResourceMap::GetAsScalar("CompositeDistribution-SolverEpsilon")))
 {
-  // We don't know if the function is thread-safe and it could be called in parallel in computePDF()
-  setParallel(false);
   setName("CompositeDistribution");
   // This method check everything and call the update() method.
   setFunctionAndAntecedent(function, antecedent);
@@ -129,9 +126,44 @@ void CompositeDistribution::setFunctionAndAntecedent(const Function & function,
   antecedent_ = antecedent;
   isAlreadyComputedMean_ = false;
   isAlreadyComputedCovariance_ = false;
+  setParallel(function.getImplementation()->isParallel() && antecedent.getImplementation()->isParallel());
   update();
 }
 
+// Structure used to wrap the gradient of the function into a Function
+class CompositeDistributionDerivativeEvaluation : public EvaluationImplementation
+{
+public:
+  explicit CompositeDistributionDerivativeEvaluation(const Function & function)
+    : EvaluationImplementation()
+    , function_(function)
+  {}
+
+  CompositeDistributionDerivativeEvaluation * clone() const override
+  {
+    return new CompositeDistributionDerivativeEvaluation(*this);
+  }
+
+  Point operator()(const Point & point) const override
+  {
+    Point value(1, function_.gradient(point)(0, 0));
+    return value;
+  }
+
+  UnsignedInteger getInputDimension() const override
+  {
+    return function_.getInputDimension();
+  }
+
+  UnsignedInteger getOutputDimension() const override
+  {
+    return 1;
+  }
+private:
+  const Function & function_;
+};
+
+
 /* Compute all the derivative attributes */
 void CompositeDistribution::update()
 {
@@ -153,8 +185,7 @@ void CompositeDistribution::update()
   Scalar fMin = values_[0];
   Scalar fMax = values_[0];
   const UnsignedInteger n = ResourceMap::GetAsUnsignedInteger("CompositeDistribution-StepNumber");
-  const DerivativeWrapper derivativeWrapper(function_);
-  const Function derivative(bindMethod<DerivativeWrapper, Point, Point>(derivativeWrapper, &DerivativeWrapper::computeDerivative, 1, 1));
+  const Function derivative(new CompositeDistributionDerivativeEvaluation(function_));
   Scalar a = xMin;
   Scalar fpA = -1.0;
   try
@@ -488,6 +519,51 @@ Bool CompositeDistribution::isDiscrete() const
   return antecedent_.isDiscrete();
 }
 
+// Structure used to compute shifted moments
+class CompositeDistributionShiftedMomentEvaluation : public EvaluationImplementation
+{
+public:
+  CompositeDistributionShiftedMomentEvaluation(const UnsignedInteger n,
+      const Scalar shift,
+      const Distribution & antecedent,
+      const Function & function)
+    : EvaluationImplementation()
+    , n_(n)
+    , shift_(shift)
+    , antecedent_(antecedent)
+    , function_(function)
+  {}
+
+  CompositeDistributionShiftedMomentEvaluation * clone() const override
+  {
+    return new CompositeDistributionShiftedMomentEvaluation(*this);
+  }
+
+  Point operator()(const Point & point) const override
+  {
+    const Scalar y = function_(point)[0];
+    const Scalar power = std::pow(y - shift_, static_cast<Scalar>(n_));
+    const Scalar pdf = antecedent_.computePDF(point);
+    const Scalar value = power * pdf;
+    return Point(1, value);
+  };
+
+  UnsignedInteger getInputDimension() const override
+  {
+    return 1;
+  }
+  UnsignedInteger getOutputDimension() const override
+  {
+    return 1;
+  }
+private:
+  UnsignedInteger n_ = 0;
+  Scalar shift_ = 0.0;
+  Distribution antecedent_;
+  Function function_;
+};
+
+
 /* Compute the shifted moments of the distribution */
 Point CompositeDistribution::computeShiftedMomentContinuous(const UnsignedInteger n,
     const Point & shift) const
@@ -497,8 +573,7 @@ Point CompositeDistribution::computeShiftedMomentContinuous(const UnsignedIntege
   Point moment(1);
   // For each component
   GaussKronrod algo;
-  const CompositeDistributionShiftedMomentWrapper kernel(n, shift[0], this);
-  const Function integrand(bindMethod<CompositeDistributionShiftedMomentWrapper, Point, Point>(kernel, &CompositeDistributionShiftedMomentWrapper::computeShiftedMomentKernel, 1, 1));
+  const Function integrand(new CompositeDistributionShiftedMomentEvaluation(n, shift[0], antecedent_, function_));
   const Scalar a = antecedent_.getRange().getLowerBound()[0];
   const Scalar b = antecedent_.getRange().getUpperBound()[0];
   moment[0] = algo.integrate(integrand, Interval(a, b))[0];
