@@ -2,7 +2,7 @@
 /**
  *  @brief OpenTURNS wrapper to a library of special functions
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -1437,7 +1437,7 @@ Scalar DistFunc::qPoisson(const Scalar lambda,
   const Scalar cp = std::min(std::max(p, SpecFunc::MinScalar), 1.0 - SpecFunc::ScalarEpsilon);
 #ifdef OPENTURNS_HAVE_BOOST
   typedef boost::math::policies::policy< boost::math::policies::discrete_quantile<boost::math::policies::integer_round_up> > integer_round_up;
-  typedef boost::math::policies::policy< boost::math::policies::discrete_quantile<boost::math::policies::integer_round_up> > integer_round_down;
+  typedef boost::math::policies::policy< boost::math::policies::discrete_quantile<boost::math::policies::integer_round_down> > integer_round_down;
   if (tail)
     return boost::math::quantile(complement(boost::math::poisson_distribution<Scalar, integer_round_down >(lambda), cp));
   else
@@ -1601,6 +1601,16 @@ Point DistFunc::pStudent(const Scalar nu,
     result[i] = StudentFunctions::StudentCDF(nu, x[i], tail);
   return result;
 }
+
+Scalar DistFunc::pStudent2D(const Scalar nu,
+                            const Scalar x1,
+                            const Scalar x2,
+                            const Scalar rho,
+                            const Bool tail)
+{
+  return StudentFunctions::Student2DCDF(nu, x1, x2, rho, tail);
+}
+
 /* CDF inverse */
 Scalar DistFunc::qStudent(const Scalar nu,
                           const Scalar p,
