@@ -2,7 +2,7 @@
 /**
  *  @brief The Normal distribution
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -111,10 +111,17 @@ Normal::Normal(const Point & mean,
   {
     const Scalar cii = C(i, i);
     if (!(cii > 0.0))
-      throw InvalidArgumentException(HERE) << "Diagonal elements of covariance matrix must be strictly positive";
-    sigma[i] = std::sqrt(cii);
-    for (UnsignedInteger j = 0; j < i; ++ j)
-      R(i, j) = C(i, j) / (sigma[i] * sigma[j]);
+    {
+      sigma[i] = 0.0;
+      // throw InvalidArgumentException(HERE) << "Diagonal elements of covariance matrix must be strictly positive";
+    }
+    else
+    {
+      sigma[i] = std::sqrt(cii);
+      for (UnsignedInteger j = 0; j < i; ++ j)
+        if (sigma[j] > 0.0)
+          R(i, j) = C(i, j) / (sigma[i] * sigma[j]);
+    }
   }
   *this = Normal(mean, sigma, R);
 }
@@ -159,10 +166,9 @@ Point Normal::getRealization() const
 {
   const UnsignedInteger dimension = getDimension();
   if (dimension == 1) return Point(1, mean_[0] + sigma_[0] * DistFunc::rNormal());
-  Point value(dimension);
   // First, a realization of independent standard coordinates
-  for (UnsignedInteger i = 0; i < dimension; i++) value[i] = DistFunc::rNormal();
-  // Then, transform the independent standard coordinates into the needed ones */
+  Point value(DistFunc::rNormal(dimension));
+  // Then, transform the independent standard coordinates into the needed ones
   if (hasIndependentCopula_)
   {
     for (UnsignedInteger i = 0; i < dimension; i++)
@@ -407,6 +413,10 @@ Scalar Normal::computeComplementaryCDF(const Point & point) const
 /* Compute the entropy of the distribution */
 Scalar Normal::computeEntropy() const
 {
+  // The entropy is equal to (see https://statproofbook.github.io/P/mvn-dent):
+  // (dim/2)*(log(2pi))+(1/2)*log(det(Sigma))+(1/2)*dim
+  // We reuse the normalization factors to avoid the recomputation of
+  // the most costly parts:
   // EllipticalDistribution::normalizationFactor_ == 1/sqrt(det(Sigma))
   // logNormalizationFactor_ == log(1/sqrt(2*Pi)^dim)
   return 0.5 * getDimension() - std::log(EllipticalDistribution::normalizationFactor_) - logNormalizationFactor_;
@@ -563,10 +573,8 @@ Scalar Normal::computeConditionalPDF(const Scalar x,
   Scalar meanRos = 0.0;
   const Scalar sigmaRos = 1.0 / inverseCholesky_(conditioningDimension, conditioningDimension);
   for (UnsignedInteger i = 0; i < conditioningDimension; ++i)
-  {
-    meanRos += inverseCholesky_(conditioningDimension, i) / std::sqrt(sigma_[i]) * (y[i] - mean_[i]);
-  }
-  meanRos = mean_[conditioningDimension] - sigmaRos * std::sqrt(sigma_[conditioningDimension]) * meanRos;
+    meanRos += inverseCholesky_(conditioningDimension, i) * (y[i] - mean_[i]);
+  meanRos = mean_[conditioningDimension] - sigmaRos * meanRos;
   const Scalar z = (x - meanRos) / sigmaRos;
   return DistFunc::dNormal(z) / sigmaRos;
 }
@@ -603,10 +611,8 @@ Scalar Normal::computeConditionalCDF(const Scalar x,
   Scalar meanRos = 0.0;
   const Scalar sigmaRos = 1.0 / inverseCholesky_(conditioningDimension, conditioningDimension);
   for (UnsignedInteger i = 0; i < conditioningDimension; ++i)
-  {
-    meanRos += inverseCholesky_(conditioningDimension, i) / std::sqrt(sigma_[i]) * (y[i] - mean_[i]);
-  }
-  meanRos = mean_[conditioningDimension] - sigmaRos * std::sqrt(sigma_[conditioningDimension]) * meanRos;
+    meanRos += inverseCholesky_(conditioningDimension, i) * (y[i] - mean_[i]);
+  meanRos = mean_[conditioningDimension] - sigmaRos * meanRos;
   return DistFunc::pNormal((x - meanRos) / sigmaRos);
 }
 
@@ -625,30 +631,25 @@ Point Normal::computeSequentialConditionalCDF(const Point & x) const
 
 /* Compute the quantile of Xi | X1, ..., Xi-1, i.e. x such that CDF(x|y) = q with x = Xi, y = (X1,...,Xi-1) */
 Scalar Normal::computeConditionalQuantile(const Scalar q,
+
     const Point & y) const
+
 {
+
   const UnsignedInteger conditioningDimension = y.getDimension();
+
   if (conditioningDimension >= getDimension()) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional quantile with a conditioning point of dimension greater or equal to the distribution dimension.";
   if ((q < 0.0) || (q > 1.0)) throw InvalidArgumentException(HERE) << "Error: cannot compute a conditional quantile for a probability level outside of [0, 1]";
   // Special case when no conditioning or independent copula
   if ((conditioningDimension == 0) || hasIndependentCopula()) return mean_[conditioningDimension] + sigma_[conditioningDimension] * DistFunc::qNormal(q);
   // General case
-  const UnsignedInteger partialDimension = conditioningDimension + 1;
-  UnsignedInteger start = 0;
-  UnsignedInteger stop = partialDimension;
-  UnsignedInteger shift = 0;
-  MatrixImplementation inverseCholeskyPartial(partialDimension, partialDimension);
-  for (UnsignedInteger i = 0; i < partialDimension; ++ i)
-  {
-    std::copy(inverseCholesky_.getImplementation()->begin() + start,
-              inverseCholesky_.getImplementation()->begin() + stop,
-              inverseCholeskyPartial.begin() + shift);
-    start += dimension_ + 1;
-    stop += dimension_;
-    shift += partialDimension + 1;
-  }
-  const Point iCholQ(inverseCholeskyPartial.solveLinearSystemTri(DistFunc::qNormal(Point(partialDimension, q))));
-  return mean_[conditioningDimension] + iCholQ[conditioningDimension];
+  Scalar meanRos = 0.0;
+  const Scalar sigmaRos = 1.0 / inverseCholesky_(conditioningDimension, conditioningDimension);
+  for (UnsignedInteger i = 0; i < conditioningDimension; ++i)
+    meanRos += inverseCholesky_(conditioningDimension, i) * (y[i] - mean_[i]);
+
+  meanRos = mean_[conditioningDimension] - sigmaRos * meanRos;
+  return meanRos + sigmaRos * DistFunc::qNormal(q);
 }
 
 Point Normal::computeSequentialConditionalQuantile(const Point & q) const
