@@ -2,7 +2,7 @@
 /**
  *  @brief A class that implements a normal copula
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,6 @@
  */
 #include <cmath>
 #include "openturns/NormalCopula.hxx"
-#include "openturns/Uniform.hxx"
 #include "openturns/IdentityMatrix.hxx"
 #include "openturns/NatafEllipticalCopulaEvaluation.hxx"
 #include "openturns/NatafEllipticalCopulaGradient.hxx"
@@ -364,7 +363,12 @@ CorrelationMatrix NormalCopula::getKendallTau() const
 /* Compute the entropy of the distribution */
 Scalar NormalCopula::computeEntropy() const
 {
-  return normal_.computeEntropy() - getDimension() * (SpecFunc::LOGSQRT2PI + 0.5);
+  // The entroy of the normal copula is the entropy of
+  // the associated standard normal distribution minus
+  // dim times the entropy of a standard univariate normal
+  // distribution, thus it is equal to (1/2)*log(|
+  //return normal_.computeEntropy() - getDimension() * (SpecFunc::LOGSQRT2PI + 0.5);
+  return -std::log(normal_.EllipticalDistribution::normalizationFactor_);
 }
 
 /* Get the Shape matrix of the copula */
@@ -378,7 +382,7 @@ Point NormalCopula::computePDFGradient(const Point & point) const
 {
   if (point.getDimension() != getDimension()) throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=" << getDimension() << ", here dimension=" << point.getDimension();
 
-  throw NotYetImplementedException(HERE) << "In NormalCopula::computePDFGradient(const Point & point) const";
+  return DistributionImplementation::computePDFGradient(point);
 }
 
 /* Get the CDF gradient of the distribution */
@@ -386,7 +390,7 @@ Point NormalCopula::computeCDFGradient(const Point & point) const
 {
   if (point.getDimension() != getDimension()) throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=" << getDimension() << ", here dimension=" << point.getDimension();
 
-  throw NotYetImplementedException(HERE) << "In NormalCopula::computeCDFGradient(const Point & point) const";
+  return DistributionImplementation::computeCDFGradient(point);
 }
 
 /* Compute the PDF of Xi | X1, ..., Xi-1. x = Xi, y = (X1,...,Xi-1)
@@ -566,9 +570,10 @@ NormalCopula::IsoProbabilisticTransformation NormalCopula::getIsoProbabilisticTr
 NormalCopula::InverseIsoProbabilisticTransformation NormalCopula::getInverseIsoProbabilisticTransformation() const
 {
   InverseIsoProbabilisticTransformation transformation;
-  transformation.setEvaluation(new InverseNatafEllipticalCopulaEvaluation(getStandardDistribution(), normal_.getCholesky()));
-  transformation.setGradient(new InverseNatafEllipticalCopulaGradient(getStandardDistribution(), normal_.getCholesky()));
-  transformation.setHessian(new InverseNatafEllipticalCopulaHessian(getStandardDistribution(), normal_.getCholesky()));
+  const TriangularMatrix cholesky(normal_.getCholesky()); // recomputed
+  transformation.setEvaluation(new InverseNatafEllipticalCopulaEvaluation(getStandardDistribution(), cholesky));
+  transformation.setGradient(new InverseNatafEllipticalCopulaGradient(getStandardDistribution(), cholesky));
+  transformation.setHessian(new InverseNatafEllipticalCopulaHessian(getStandardDistribution(), cholesky));
 
   return transformation;
 }
@@ -699,10 +704,7 @@ void NormalCopula::save(Advocate & adv) const
 {
   DistributionImplementation::save(adv);
   adv.saveAttribute( "correlation_", correlation_ );
-  adv.saveAttribute( "covariance_duplicate", covariance_ );
   adv.saveAttribute( "normal_", normal_ );
-  adv.saveAttribute( "integrationNodesNumber_duplicate", integrationNodesNumber_ );
-  adv.saveAttribute( "isAlreadyComputedCovariance_duplicate", isAlreadyComputedCovariance_ );
 }
 
 /* Method load() reloads the object from the StorageManager */
@@ -711,10 +713,7 @@ void NormalCopula::load(Advocate & adv)
   // The range is generic for all the copulas
   DistributionImplementation::load(adv);
   adv.loadAttribute( "correlation_", correlation_ );
-  adv.loadAttribute( "covariance_duplicate", covariance_ );
   adv.loadAttribute( "normal_", normal_ );
-  adv.loadAttribute( "integrationNodesNumber_duplicate", integrationNodesNumber_ );
-  adv.loadAttribute( "isAlreadyComputedCovariance_duplicate", isAlreadyComputedCovariance_ );
   computeRange();
 }
 
