@@ -2,7 +2,7 @@
 /**
  *  @brief The PointConditionalDistribution distribution
  *
- *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -25,9 +25,11 @@
 #include "openturns/ParametricFunction.hxx"
 #include "openturns/PersistentObjectFactory.hxx"
 #include "openturns/ResourceMap.hxx"
+#include "openturns/RandomGenerator.hxx"
 #include "openturns/SimplicialCubature.hxx"
 #include "openturns/CubaIntegration.hxx"
 #include "openturns/Tuples.hxx"
+#include "openturns/ArchimedeanCopula.hxx"
 #include "openturns/Beta.hxx"
 #include "openturns/BlockIndependentCopula.hxx"
 #include "openturns/BlockIndependentDistribution.hxx"
@@ -36,14 +38,14 @@
 #include "openturns/KernelMixture.hxx"
 #include "openturns/Mixture.hxx"
 #include "openturns/Normal.hxx"
+#include "openturns/NormalCopula.hxx"
 #include "openturns/Student.hxx"
-#include "openturns/OptimizationAlgorithm.hxx"
-#include "openturns/RandomGenerator.hxx"
 #include "openturns/DomainEvent.hxx"
 #include "openturns/PlatformInfo.hxx"
 #include "openturns/GaussKronrod.hxx"
 #include "openturns/DistFunc.hxx"
 #include "openturns/SobolSequence.hxx"
+#include "openturns/MarginalTransformationEvaluation.hxx"
 
 BEGIN_NAMESPACE_OPENTURNS
 
@@ -68,6 +70,7 @@ PointConditionalDistribution::PointConditionalDistribution(const Distribution &
   , distribution_(distribution)
 {
   setName("PointConditionalDistribution");
+  setParallel(distribution.getImplementation()->isParallel());
   const UnsignedInteger fullDimension = distribution.getDimension();
 
   if (conditioningIndices.getSize() != conditioningValues.getSize())
@@ -79,7 +82,10 @@ PointConditionalDistribution::PointConditionalDistribution(const Distribution &
   Indices permutation(conditioningIndices.getSize());
   permutation.fill();
   std::sort(permutation.begin(), permutation.end(),
-            [&](const UnsignedInteger & a, const UnsignedInteger & b) { return (conditioningIndices[a] < conditioningIndices[b]); });
+            [&](const UnsignedInteger & a, const UnsignedInteger & b)
+  {
+    return (conditioningIndices[a] < conditioningIndices[b]);
+  });
   conditioningIndices_.resize(conditioningIndices.getSize());
   conditioningValues_.resize(conditioningIndices.getSize());
   for (UnsignedInteger i = 0; i < conditioningIndices.getSize(); ++ i)
@@ -88,6 +94,37 @@ PointConditionalDistribution::PointConditionalDistribution(const Distribution &
     conditioningValues_[i] = conditioningValues[permutation[i]];
   }
 
+  // check if we want & can use generic conditional methods
+  // first, check ResourceMap
+  useGenericConditionalMethods_ = ResourceMap::GetAsBool("PointConditionalDistribution-UseGenericConditionalMethods");
+  if (useGenericConditionalMethods_)
+  {
+    // then, check if we are in a case where it *could be* possible to use these methods
+    const UnsignedInteger conditioningSize = conditioningIndices.getSize();
+    // is the point conditional distribution univariate?
+    if (conditioningSize == fullDimension - 1)
+    {
+      // If we are in the case of a symmetric bivariate copula (elliptical or archimedean), then X0|X1=x is the same as X1|X0=x
+      // but only the last one allows for the use of generic methods. Change the conditioning order in this case
+      if ((conditioningSize > 0) && (distribution.getDimension() == 2) && distribution.isCopula() && (conditioningIndices[0] == 1))
+      {
+        // elliptical case: swap the conditioning
+        if (distribution.hasIndependentCopula())
+          conditioningIndices_[0] = 0;
+        else
+        {
+          const ArchimedeanCopula * p_archimedean = dynamic_cast< ArchimedeanCopula * >(distribution.getCopula().getImplementation().get());
+          // archimedean case: swap the conditioning
+          if (p_archimedean)
+            conditioningIndices_[0] = 0;
+        } // No elliptical copula
+      } // bivariate copula to swap
+      // Now, check if the last component is the one conditioned by the others
+      useGenericConditionalMethods_ = (conditioningSize > 0) && (conditioningIndices[conditioningSize - 1] == (fullDimension - 2));
+    } // conditioningSize == fullDimension - 1
+    else
+      useGenericConditionalMethods_ = false;
+  } // useGenericConditionalMethods_ from ResourceMap
   // it is ok to condition continuous marginals by a discrete one and vice-versa
   const Distribution marginalConditioned(distribution.getMarginal(conditioningIndices.complement(fullDimension)));
   if (!marginalConditioned.isDiscrete() && !marginalConditioned.isContinuous())
@@ -308,17 +345,21 @@ void PointConditionalDistribution::update()
   setDescription(distribution_.getDescription().select(nonConditioningIndices_));
 
   // enable simplified path
-  useSimplifiedVersion_ = hasSimplifiedVersion(simplifiedVersion_);
+  if (ResourceMap::GetAsBool("PointConditionalDistribution-UseSimplifiedVersion"))
+    useSimplifiedVersion_ = hasSimplifiedVersion(simplifiedVersion_);
+  else useSimplifiedVersion_ = false;
   LOGDEBUG(OSS() << "useSimplifiedVersion_=" << useSimplifiedVersion_);
+  if (useSimplifiedVersion_)
+    setParallel(simplifiedVersion_.getImplementation()->isParallel());
 
   // We can postpone the computation of the normalization factor here as we will not need it if there is a simplified version (and it can be costly due to the marginal extraction)
   if (!useSimplifiedVersion_)
-    {
-      if (conditioningIndices_.getSize())
-	logNormalizationFactor_ = distribution_.getMarginal(conditioningIndices_).computeLogPDF(conditioningValues_);
-      if (!(logNormalizationFactor_ > std::log(getPDFEpsilon())))
-	throw InvalidArgumentException(HERE) << "Conditioning vector log PDF value is too low (" << logNormalizationFactor_ << ")";
-    }
+  {
+    if (conditioningIndices_.getSize())
+      logNormalizationFactor_ = distribution_.getMarginal(conditioningIndices_).computeLogPDF(conditioningValues_);
+    if (!SpecFunc::IsNormal(logNormalizationFactor_))
+      throw InvalidArgumentException(HERE) << "Conditioning vector log PDF value is too low (" << logNormalizationFactor_ << ")";
+  }
 
   // cache marginal for reuse
   if (!useSimplifiedVersion_)
@@ -329,7 +370,7 @@ void PointConditionalDistribution::update()
   isAlreadyComputedCovariance_ = false;
 
   // initialize alias method
-  if (!useSimplifiedVersion_ && isDiscrete())
+  if (!useSimplifiedVersion_ && !useGenericConditionalMethods_ && isDiscrete())
   {
     support_ = getSupport();
     probabilities_ = computePDF(support_).getImplementation()->getData();
@@ -339,103 +380,16 @@ void PointConditionalDistribution::update()
   // initialize ratio of uniforms method, see https://en.wikipedia.org/wiki/Ratio_of_uniforms
   // r_ is a free parameter, could be optimized to maximize the acceptance ratio
   const UnsignedInteger dimension = getDimension();
-  if (!useSimplifiedVersion_ && isContinuous() && (dimension <= ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-SmallDimension"))
-    && ResourceMap::GetAsBool("PointConditionalDistribution-InitializeSampling"))
+  if (!useSimplifiedVersion_ && !useGenericConditionalMethods_ && isContinuous() && (dimension <= ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-SmallDimension"))
+      && ResourceMap::GetAsBool("PointConditionalDistribution-InitializeSampling"))
   {
-    const Interval bounds(getRange());
-    const Point lb(bounds.getLowerBound());
-    const Point ub(bounds.getUpperBound());
-
-    // find a feasible starting point
-    SobolSequence sequence(dimension);
-    Point start;
-    const UnsignedInteger candidateNumber = ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-RatioUniformCandidateNumber");
-    for (UnsignedInteger k = 0; k < candidateNumber; ++ k)
-    {
-      Point candidate(sequence.generate());
-      for (UnsignedInteger j = 0; j < dimension; ++ j)
-        candidate[j] = lb[j] + candidate[j] * (ub[j] - lb[j]);
-      if (computePDF(candidate) > 0.0)
-      {
-        start = candidate;
-        break;
-      }
-    }
-    if (!start.getDimension())
-      throw InternalException(HERE) << "Could not find a feasible starting point to initialize ration of uniforms U sup";
-
-    // First, the upper bound on U
-    const Function objectiveU(new PointConditionalDistributionUBoundEvaluation(*this, r_));
-    OptimizationProblem problemU(objectiveU);
-    problemU.setMinimization(false);
-    problemU.setBounds(bounds);
-    OptimizationAlgorithm algo(OptimizationAlgorithm::GetByName(ResourceMap::GetAsString("PointConditionalDistribution-OptimizationAlgorithm")));
-    algo.setProblem(problemU);
-    algo.setStartingPoint(start);
-    algo.run();
-    supU_ = std::exp(algo.getResult().getOptimalValue()[0]);
-    LOGDEBUG(OSS() << "supU_=" << supU_ << " u*=" << algo.getResult().getOptimalPoint());
-
-    // Second, the lower and upper bounds on V
-    const Function objectiveV(new PointConditionalDistributionVBoundEvaluation(*this, r_));
-    infV_.resize(dimension);
-    supV_.resize(dimension);
-    const Point zero(dimension, 0.0);
-    for (UnsignedInteger i = 0; i < dimension; ++ i)
-    {
-      const Function objectiveVI(objectiveV.getMarginal(i));
-      OptimizationProblem problemVI(objectiveVI);
-      problemVI.setMinimization(false);
-      if (ub[i] > 0.0)
-      {
-        // find a feasible starting point in [0, ub]
-        start.clear();
-        for (UnsignedInteger k = 0; k < candidateNumber; ++ k)
-        {
-          Point candidate(sequence.generate());
-          for (UnsignedInteger j = 0; j < dimension; ++ j)
-            candidate[j] = candidate[j] * ub[j];
-          if (computePDF(candidate) > 0.0)
-          {
-            start = candidate;
-            break;
-          }
-        }
-        if (!start.getDimension())
-          throw InternalException(HERE) << "Could not find a feasible starting point to initialize ration of uniforms V sup";
-        problemVI.setBounds(Interval(zero, ub));
-        algo.setProblem(problemVI);
-        algo.setStartingPoint(start);
-        algo.run();
-        supV_[i] = std::exp(algo.getResult().getOptimalValue()[0]);
-        LOGDEBUG(OSS() << "supV_[" << i << "]=" << supV_[i] << " v*=" << algo.getResult().getOptimalPoint());
-      }
-      if (lb[i] < 0.0)
-      {
-        // find a feasible starting point in [lb, 0]
-        start.clear();
-        for (UnsignedInteger k = 0; k < candidateNumber; ++ k)
-        {
-          Point candidate(sequence.generate());
-          for (UnsignedInteger j = 0; j < dimension; ++ j)
-            candidate[j] = candidate[j] * lb[j];
-          if (computePDF(candidate) > 0.0)
-          {
-            start = candidate;
-            break;
-          }
-        }
-        if (!start.getDimension())
-          throw InternalException(HERE) << "Could not find a feasible starting point to initialize ration of uniforms V inf";
-        problemVI.setBounds(Interval(lb, zero));
-        algo.setProblem(problemVI);
-        algo.setStartingPoint(start);
-        algo.run();
-        infV_[i] = -std::exp(algo.getResult().getOptimalValue()[0]);
-        LOGDEBUG(OSS() << "infV_[" << i << "]=" << infV_[i] << " v*=" << algo.getResult().getOptimalPoint());
-      }
-    }
-  }
+    // initialize ratio of uniforms method, see https://en.wikipedia.org/wiki/Ratio_of_uniforms
+    // r_ is a free parameter, could be optimized to maximize the acceptance ratio
+    sampler_ = RatioOfUniforms();
+    sampler_.setOptimizationAlgorithm(OptimizationAlgorithm::GetByName(ResourceMap::GetAsString("PointConditionalDistribution-OptimizationAlgorithm")));
+    sampler_.setCandidateNumber(ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-RatioUniformCandidateNumber"));
+    sampler_.setLogUnscaledPDFAndRange(getLogPDF(), getRange(), true);
+  } // isContinuous()
 
   if (!useSimplifiedVersion_ && ResourceMap::GetAsBool("PointConditionalDistribution-InitializeTransformation"))
   {
@@ -454,20 +408,24 @@ void PointConditionalDistribution::update()
 }
 
 
-Point PointConditionalDistribution::decompose(CovarianceMatrix & C) const
+Point PointConditionalDistribution::decompose(const Distribution & distribution,
+    const Indices & conditioningIndices,
+    const Indices & nonConditioningIndices,
+    const Point & conditioningValues,
+    CovarianceMatrix & C) const
 {
-  const Point mu(distribution_.getMean());
-  const CovarianceMatrix cov(distribution_.getCovariance());
-  const CovarianceMatrix cxx(distribution_.getMarginal(nonConditioningIndices_).getCovariance());
-  const CovarianceMatrix cyy(distribution_.getMarginal(conditioningIndices_).getCovariance());
-  const Point mux = mu.select(nonConditioningIndices_);
-  const Point muy = mu.select(conditioningIndices_);
-  Matrix cxy(conditioningIndices_.getSize(), nonConditioningIndices_.getSize());
-  for (UnsignedInteger i = 0; i < conditioningIndices_.getSize(); ++ i)
-    for (UnsignedInteger j = 0; j < nonConditioningIndices_.getSize(); ++ j)
-      cxy(i, j) = cov(conditioningIndices_[i], nonConditioningIndices_[j]);
+  const Point mu(distribution.getMean());
+  const CovarianceMatrix cov(distribution.getCovariance());
+  const CovarianceMatrix cxx(distribution.getMarginal(nonConditioningIndices).getCovariance());
+  const CovarianceMatrix cyy(distribution.getMarginal(conditioningIndices).getCovariance());
+  const Point mux = mu.select(nonConditioningIndices);
+  const Point muy = mu.select(conditioningIndices);
+  Matrix cxy(conditioningIndices.getSize(), nonConditioningIndices.getSize());
+  for (UnsignedInteger i = 0; i < conditioningIndices.getSize(); ++ i)
+    for (UnsignedInteger j = 0; j < nonConditioningIndices.getSize(); ++ j)
+      cxy(i, j) = cov(conditioningIndices[i], nonConditioningIndices[j]);
   // here we could get the inverse cholesky from Elliptical but we cannot access the covariance scaling factor
-  const Point muConditional(mux + cxy.transpose() * cyy.solveLinearSystem(conditioningValues_ - muy));
+  const Point muConditional(mux + cxy.transpose() * cyy.solveLinearSystem(conditioningValues - muy));
   C = CovarianceMatrix((cxx - (cxy.transpose() * cyy.solveLinearSystem(cxy))).getImplementation());
   return muConditional;
 }
@@ -475,9 +433,6 @@ Point PointConditionalDistribution::decompose(CovarianceMatrix & C) const
 /* Get the simplified version */
 Bool PointConditionalDistribution::hasSimplifiedVersion(Distribution & simplified) const
 {
-  if (!ResourceMap::GetAsBool("PointConditionalDistribution-UseSimplifiedVersion"))
-    return false;
-
   // no conditioning (empty point)
   if (getDimension() == distribution_.getDimension())
   {
@@ -492,50 +447,61 @@ Bool PointConditionalDistribution::hasSimplifiedVersion(Distribution & simplifie
     return true;
   }
 
-  // Normal
-  Normal *p_normal = dynamic_cast<Normal *>(distribution_.getImplementation().get());
-  if (p_normal)
+  // The elliptical distributions
+  if (distribution_.isElliptical())
   {
-    CovarianceMatrix C;
-    const Point mu(decompose(C));
-    simplified = Normal(mu, C);
-    return true;
-  }
+    const String standardSpace(distribution_.getStandardRepresentative().getImplementation()->getClassName());
+    // Normal case
+    if (standardSpace == "Normal")
+    {
+      CovarianceMatrix C;
+      const Point mu(decompose(distribution_, conditioningIndices_, nonConditioningIndices_, conditioningValues_, C));
+      simplified = Normal(mu, C);
+      return true;
+    } // Normal case
 
-  // Student
-  Student *p_student = dynamic_cast<Student *>(distribution_.getImplementation().get());
-  if (p_student)
-  {
-    CovarianceMatrix C;
-    const Point mu(decompose(C));
-    const Point mY(conditioningValues_ - mu.select(conditioningIndices_));
-    const Scalar dy = mY.dot(distribution_.getMarginal(conditioningIndices_).getCovariance().solveLinearSystem(mY));
-    const Scalar nu = p_student->getNu();
-    const UnsignedInteger py = conditioningIndices_.getSize();
-    C = CovarianceMatrix((C * std::sqrt((nu + dy) / (nu + py))).getImplementation());
-    simplified = Student(nu + py, mu, C);
-    return true;
-  }
+    // Student case
+    if (standardSpace == "Student")
+    {
+      CovarianceMatrix C;
+      const Point mu(decompose(distribution_, conditioningIndices_, nonConditioningIndices_, conditioningValues_, C));
+      const Point mY(conditioningValues_ - mu.select(conditioningIndices_));
+      const Scalar dy = mY.dot(distribution_.getMarginal(conditioningIndices_).getCovariance().solveLinearSystem(mY));
+      const Student *p_student = dynamic_cast<Student *>(distribution_.getImplementation().get());
+      const Scalar nu = p_student->getNu();
+      const UnsignedInteger py = conditioningIndices_.getSize();
+      C = CovarianceMatrix((C * std::sqrt((nu + dy) / (nu + py))).getImplementation());
+      simplified = Student(nu + py, mu, C);
+      return true;
+    } // Student case
+  } // Elliptical
 
   // Mixture
-  Mixture *p_mixture = dynamic_cast<Mixture *>(distribution_.getImplementation().get());
+  const Mixture *p_mixture = dynamic_cast<Mixture *>(distribution_.getImplementation().get());
   if (p_mixture)
   {
     Collection<Distribution> atoms(p_mixture->getDistributionCollection());
     const UnsignedInteger atomsNumber = atoms.getSize();
-    Point newWeights(p_mixture->getWeights());
-    Collection<Distribution> newAtoms(atomsNumber);
+    const Point weights(p_mixture->getWeights());
+    Point newWeights;
+    Collection<Distribution> newAtoms;
     for (UnsignedInteger i = 0; i < atomsNumber; ++i)
     {
-      newWeights[i] *= atoms[i].getMarginal(conditioningIndices_).computePDF(conditioningValues_);
-      newAtoms[i] = PointConditionalDistribution(atoms[i], conditioningIndices_, conditioningValues_);
-    }
+      const Scalar w = atoms[i].getMarginal(conditioningIndices_).computePDF(conditioningValues_);
+      // Add only atoms with nonzero distribution
+      if (w > 0.0)
+      {
+        newWeights.add(weights[i] * w);
+        const PointConditionalDistribution atom(atoms[i], conditioningIndices_, conditioningValues_);
+        newAtoms.add(atom.getSimplifiedVersion());
+      }
+    } // for i
     simplified = Mixture(newAtoms, newWeights);
     return true;
   }
 
   // Kernel mixture
-  KernelMixture *p_kernel_mixture = dynamic_cast<KernelMixture *>(distribution_.getImplementation().get());
+  const KernelMixture *p_kernel_mixture = dynamic_cast<KernelMixture *>(distribution_.getImplementation().get());
   if (p_kernel_mixture)
   {
     const Distribution kernel(p_kernel_mixture->getKernel());
@@ -569,41 +535,45 @@ Bool PointConditionalDistribution::hasSimplifiedVersion(Distribution & simplifie
   }
 
   // EmpiricalBernsteinCopula
-  EmpiricalBernsteinCopula *p_empirical_bernstein_copula = dynamic_cast<EmpiricalBernsteinCopula *>(distribution_.getImplementation().get());
+  const EmpiricalBernsteinCopula *p_empirical_bernstein_copula = dynamic_cast<EmpiricalBernsteinCopula *>(distribution_.getImplementation().get());
   if (p_empirical_bernstein_copula)
   {
     const Sample copulaSample(p_empirical_bernstein_copula->getCopulaSample());
     const UnsignedInteger sampleSize = copulaSample.getSize();
     const UnsignedInteger binNumber = p_empirical_bernstein_copula->getBinNumber();
-    Collection<Distribution> atoms(sampleSize);
-    Point weights(sampleSize, 0.0);
+    Collection<Distribution> atoms;
+    Point weights;
     const UnsignedInteger dimension = getDimension();
     const UnsignedInteger conditioningDimension = conditioningIndices_.getSize();
     for (UnsignedInteger i = 0; i < sampleSize; ++i)
     {
-      Collection<Distribution> atomComponents(dimension);
-      for (UnsignedInteger j = 0; j < dimension; ++j)
-      {
-        const UnsignedInteger newJ = nonConditioningIndices_[j];
-        const Scalar r = std::ceil(binNumber * copulaSample(i, newJ));
-        atomComponents[j] = Beta(r, binNumber - r + 1.0, 0.0, 1.0);
-      } // j
-      atoms[i] = JointDistribution(atomComponents);
+      Scalar logWi = 0.0;
       for (UnsignedInteger j = 0; j < conditioningDimension; ++j)
       {
         const UnsignedInteger newJ = conditioningIndices_[j];
         const Scalar r = std::ceil(binNumber * copulaSample(i, newJ));
         const Scalar xJ = conditioningValues_[j];
-        weights[i] += -SpecFunc::LogBeta(r, binNumber - r + 1.0) + (r - 1.0) * std::log(xJ) + (binNumber - r) * std::log1p(-xJ);
+        logWi += -SpecFunc::LogBeta(r, binNumber - r + 1.0) + (r - 1.0) * std::log(xJ) + (binNumber - r) * std::log1p(-xJ);
       } // j
-      weights[i] = std::exp(weights[i]);
+      if (SpecFunc::IsNormal(logWi))
+      {
+        weights.add(std::exp(logWi));
+        Collection<Distribution> atomComponents(dimension);
+        for (UnsignedInteger j = 0; j < dimension; ++j)
+        {
+          const UnsignedInteger newJ = nonConditioningIndices_[j];
+          const Scalar r = std::ceil(binNumber * copulaSample(i, newJ));
+          atomComponents[j] = Beta(r, binNumber - r + 1.0, 0.0, 1.0);
+        } // j
+        atoms.add(JointDistribution(atomComponents));
+      } // isNormal
     } // i
     simplified = Mixture(atoms, weights);
     return true;
   }
 
   // BlockIndependentDistribution
-  BlockIndependentDistribution *p_block_independent_distribution = dynamic_cast<BlockIndependentDistribution *>(distribution_.getImplementation().get());
+  const BlockIndependentDistribution *p_block_independent_distribution = dynamic_cast<BlockIndependentDistribution *>(distribution_.getImplementation().get());
   if (p_block_independent_distribution)
   {
     dispatchConditioning(p_block_independent_distribution->getDistributionCollection(), simplified);
@@ -611,25 +581,36 @@ Bool PointConditionalDistribution::hasSimplifiedVersion(Distribution & simplifie
   }
 
   // BlockIndependentCopula
-  BlockIndependentCopula *p_block_independent_copula = dynamic_cast<BlockIndependentCopula *>(distribution_.getImplementation().get());
+  const BlockIndependentCopula *p_block_independent_copula = dynamic_cast<BlockIndependentCopula *>(distribution_.getImplementation().get());
   if (p_block_independent_copula)
   {
     dispatchConditioning(p_block_independent_copula->getCopulaCollection(), simplified);
     return true;
   }
 
-  // Joint
+  // Joint. Has we don't have an efficient PointConditionalCopula we restrict ourselve to the Bernstein copula
   JointDistribution *p_joint = dynamic_cast<JointDistribution *>(distribution_.getImplementation().get());
   if (p_joint)
   {
-    const Collection<Distribution> marginals(p_joint->getDistributionCollection());
-    Point coreConditioniningValues(conditioningIndices_.getSize());
-    for (UnsignedInteger i = 0; i < conditioningIndices_.getSize(); ++ i)
-      coreConditioniningValues[i] = marginals[conditioningIndices_[i]].computeCDF(conditioningValues_[i]);
-    const PointConditionalDistribution conditionalCore(p_joint->getCore(), conditioningIndices_, coreConditioniningValues);
-    simplified = JointDistribution(marginals.select(nonConditioningIndices_), conditionalCore);
-    return true;
-  }
+    const EmpiricalBernsteinCopula *p_bernstein = dynamic_cast<EmpiricalBernsteinCopula *>(distribution_.getCopula().getImplementation().get());
+    if (p_bernstein)
+    {
+      const Collection<Distribution> marginals(p_joint->getDistributionCollection());
+      Point coreConditioniningValues(conditioningIndices_.getSize());
+      for (UnsignedInteger i = 0; i < conditioningIndices_.getSize(); ++ i)
+      {
+        const Scalar conditioningValueI = marginals[conditioningIndices_[i]].computeCDF(conditioningValues_[i]);
+        // If the conditioning value is too close to 1 or too close to 0
+        // the conditioning of the core will fail
+        if ((conditioningValueI <= cdfEpsilon_) || (conditioningValueI >= 1.0 - cdfEpsilon_))
+          return false;
+        coreConditioniningValues[i] = conditioningValueI;
+      }
+      const PointConditionalDistribution conditionalCore(p_joint->getCore(), conditioningIndices_, coreConditioniningValues);
+      simplified = JointDistribution(marginals.select(nonConditioningIndices_), conditionalCore);
+      return true;
+    } // p_bernstein
+  } // p_joint
 
   return false;
 }
@@ -645,7 +626,80 @@ void PointConditionalDistribution::computeRange()
   if (useSimplifiedVersion_)
     setRange(simplifiedVersion_.getRange());
   else
-    setRange(distribution_.getRange().getMarginal(nonConditioningIndices_));
+  {
+    // We have three strategies:
+    // Strategy one [None], fast but risky: the conditioning has no
+    // influence on the marginal range.
+    // Strategy two [Normal], more robust: the conditioning acts on the
+    // marginal range the same way it does on a Normal with the same mean
+    // and covariance, conditioned the same way. The adapted range is the
+    // range of this conditioned distribution. It only uses the mean and
+    // the covariance of the distribution to be conditioned so it is not
+    // too costly for many distributions.
+    // Strategy three [NormalCopula], robust but slow: the conditioning acts
+    // on the marginal range the same way it acts on a distribution with the
+    // same marginals and a normal copula having the same Spearman
+    // correlation. In addition to the Spearman correlation one has to
+    // extract all the 1D marginal distributions.
+    const String adaptationMethod(ResourceMap::GetAsString("PointConditionalDistribution-RangeAdaptationMethod"));
+    // The marginal range
+    const Interval marginalRange(distribution_.getRange().getMarginal(nonConditioningIndices_));
+    // First strategy
+    if (adaptationMethod == "None")
+      setRange(marginalRange);
+    // Second and third strategies
+    else
+    {
+      // Copy to get the correct flags
+      Interval conditionedRange(marginalRange);
+      if ((adaptationMethod == "Normal") || (adaptationMethod == "NormalLinear"))
+      {
+        const Point mean(distribution_.getMean());
+        const CovarianceMatrix covariance(distribution_.getCovariance());
+        const Normal normal(mean, covariance);
+        CovarianceMatrix C;
+        const Point mu = decompose(normal, conditioningIndices_, nonConditioningIndices_, conditioningValues_, C);
+        const Normal conditionedNormal(mu, C);
+        const Interval normalConditionedRange(conditionedNormal.getRange());
+        conditionedRange.setLowerBound(normalConditionedRange.getLowerBound());
+        conditionedRange.setUpperBound(normalConditionedRange.getUpperBound());
+      } // Strategy = Normal
+      // Third strategy
+      else
+      {
+        const UnsignedInteger dimension = distribution_.getDimension();
+        const Point mean(dimension, 0.0);
+        const CorrelationMatrix Rspearman(distribution_.getSpearmanCorrelation());
+        const CovarianceMatrix covariance(NormalCopula::GetCorrelationFromSpearmanCorrelation(Rspearman));
+        const Normal normal(mean, covariance);
+        // Extract the marginal distributions
+        Collection<Distribution> marginals(dimension);
+        for (UnsignedInteger i = 0; i < dimension; ++i)
+          marginals[i] = distribution_.getMarginal(i);
+        // Compute the equivalent normal conditioning values
+        const UnsignedInteger conditioningDimension = conditioningIndices_.getSize();
+        Collection<Distribution> conditioningMarginals(conditioningDimension);
+        for (UnsignedInteger i = 0; i < conditioningDimension; ++i)
+          conditioningMarginals[i] = marginals[conditioningIndices_[i]];
+        const Point normalConditioningValues(MarginalTransformationEvaluation(conditioningMarginals, Collection<Distribution>(conditioningDimension, Normal()))(conditioningValues_));
+        CovarianceMatrix C;
+        const Point mu = decompose(normal, conditioningIndices_, nonConditioningIndices_, normalConditioningValues, C);
+        const Normal conditionedNormal(mu, C);
+        const Interval normalConditionedRange(conditionedNormal.getRange());
+        // Adapt the range using marginal quantiles
+        const Point x(normalConditionedRange.getLowerBound());
+        const Point y(normalConditionedRange.getUpperBound());
+        const UnsignedInteger conditionedDimension = nonConditioningIndices_.getSize();
+        Collection<Distribution> conditionedMarginals(conditionedDimension);
+        for (UnsignedInteger i = 0; i < conditionedDimension; ++i)
+          conditionedMarginals[i] = marginals[nonConditioningIndices_[i]];
+        const MarginalTransformationEvaluation T(Collection<Distribution>(conditionedDimension, Normal()), conditionedMarginals);
+        conditionedRange.setLowerBound(T(x));
+        conditionedRange.setUpperBound(T(y));
+        setRange(conditionedRange);
+      } // Strategy == NormalCopula
+    } // Strategy != None
+  } // !useSimplifiedVersion_
 }
 
 /* Get the support of a distribution that intersects a given interval */
@@ -673,11 +727,12 @@ Sample PointConditionalDistribution::getSupport(const Interval & interval) const
   return support;
 }
 
-/* Get one realization of the distribution */
 Point PointConditionalDistribution::getRealization() const
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.getRealization();
+  if (useGenericConditionalMethods_)
+    return Point(1, distribution_.computeConditionalQuantile(RandomGenerator::Generate(), conditioningValues_));
 
   if (isDiscrete())
   {
@@ -686,29 +741,31 @@ Point PointConditionalDistribution::getRealization() const
   }
 
   const UnsignedInteger dimension = getDimension();
-  if (isContinuous() && dimension <= ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-SmallDimension"))
-  {
-    if (!infV_.getSize())
-      throw InvalidArgumentException(HERE) << "Sampling was not initialized";
-
-    // Now, the sampling using rejection
-    Bool accepted = false;
-    Point result(dimension);
-    while (!accepted)
-    {
-      const Scalar u = supU_ * RandomGenerator::Generate();
-      const Scalar ur = std::pow(u, r_);
-      for (UnsignedInteger i = 0; i < dimension; ++ i)
-        result[i] = (infV_[i] + (supV_[i] - infV_[i]) * RandomGenerator::Generate()) / ur;
-      accepted = (1.0 + r_ * dimension) * std::log(u) <= computeLogPDF(result);
-    }
-    return result;
-  }
+  if (isContinuous() && dimension <= ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-SmallDimension") && sampler_.isInitialized())
+    return sampler_.getRealization();
 
   return DistributionImplementation::getRealization();
 }
 
 
+Sample PointConditionalDistribution::getSample(const UnsignedInteger size) const
+{
+  if (useSimplifiedVersion_)
+    return simplifiedVersion_.getSample(size);
+
+  if (isDiscrete())
+  {
+    const Indices indices(DistFunc::rDiscrete(base_, alias_, size));
+    return support_.select(indices);
+  }
+
+  const UnsignedInteger dimension = getDimension();
+  if (isContinuous() && dimension <= ResourceMap::GetAsUnsignedInteger("PointConditionalDistribution-SmallDimension") && sampler_.isInitialized())
+    return sampler_.getSample(size);
+
+  return DistributionImplementation::getSample(size);
+}
+
 Point PointConditionalDistribution::expandPoint(const Point & point) const
 {
   const UnsignedInteger dimension = getDimension();
@@ -735,8 +792,9 @@ Scalar PointConditionalDistribution::computeLogPDF(const Point & point) const
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.computeLogPDF(point);
-  else
-    return distribution_.computeLogPDF(expandPoint(point)) - logNormalizationFactor_;
+  if (useGenericConditionalMethods_)
+    return std::log(distribution_.computeConditionalPDF(point[0], conditioningValues_));
+  return distribution_.computeLogPDF(expandPoint(point)) - logNormalizationFactor_;
 }
 
 /* Get the PDF of the distribution */
@@ -744,8 +802,9 @@ Scalar PointConditionalDistribution::computePDF(const Point & point) const
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.computePDF(point);
-  else
-    return std::exp(computeLogPDF(point));
+  if (useGenericConditionalMethods_)
+    return distribution_.computeConditionalPDF(point[0], conditioningValues_);
+  return std::exp(computeLogPDF(point));
 }
 
 /* Get the CDF of the distribution */
@@ -753,7 +812,8 @@ Scalar PointConditionalDistribution::computeCDF(const Point & point) const
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.computeCDF(point);
-
+  if (useGenericConditionalMethods_)
+    return distribution_.computeConditionalCDF(point[0], conditioningValues_);
   return computeProbability(Interval(getRange().getLowerBound(), point));
 }
 
@@ -762,6 +822,8 @@ Scalar PointConditionalDistribution::computeProbability(const Interval & interva
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.computeProbability(interval);
+  if (useGenericConditionalMethods_)
+    return distribution_.computeConditionalCDF(interval.getUpperBound()[0], conditioningValues_) - distribution_.computeConditionalCDF(interval.getLowerBound()[0], conditioningValues_);
 
   const UnsignedInteger dimension = getDimension();
   if (interval.getDimension() != dimension)
@@ -778,7 +840,7 @@ Scalar PointConditionalDistribution::computeProbability(const Interval & interva
     if (isContinuous())
     {
       // Build the relevant parametric function to be integrated over the remaining parameters
-      const ParametricFunction kernel(PDFWrapper(distribution_.getImplementation()->clone()), conditioningIndices_, conditioningValues_);
+      const ParametricFunction kernel(distribution_.getPDF(), conditioningIndices_, conditioningValues_);
       probability = integrationAlgorithm_.integrate(kernel, intersection)[0] / std::exp(logNormalizationFactor_);
     }
     else
@@ -792,6 +854,8 @@ Scalar PointConditionalDistribution::computeScalarQuantile(const Scalar prob, co
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.computeScalarQuantile(prob, tail);
+  if (useGenericConditionalMethods_)
+    return distribution_.computeConditionalQuantile(tail ? 1.0 - prob : prob, conditioningValues_);
 
   return DistributionImplementation::computeScalarQuantile(prob, tail);
 }
@@ -800,6 +864,8 @@ Point PointConditionalDistribution::computeQuantile(const Scalar prob, const Boo
 {
   if (useSimplifiedVersion_)
     return simplifiedVersion_.computeQuantile(prob, tail);
+  if (useGenericConditionalMethods_)
+    return Point(1, distribution_.computeConditionalQuantile(tail ? 1.0 - prob : prob, conditioningValues_));
 
   return DistributionImplementation::computeQuantile(prob, tail);
 }
@@ -948,7 +1014,9 @@ Distribution PointConditionalDistribution::getMarginal(const Indices & indices)
 Point PointConditionalDistribution::getParameter() const
 {
   // parameters of the conditioned then conditioning values
-  Point parameter(distribution_.getParameter());
+  Point parameter;
+  if (ResourceMap::GetAsBool("PointConditionalDistribution-UseFullParameters"))
+    parameter = distribution_.getParameter();
   parameter.add(conditioningValues_);
   return parameter;
 }
@@ -960,21 +1028,26 @@ void PointConditionalDistribution::setParameter(const Point & parameter)
     throw InvalidArgumentException(HERE) << "Parameter size should be " << currentParameter.getSize() << " got " << parameter.getSize();
 
   // set parameters of the conditioned
-  Point conditionedParameter(distribution_.getParameter().getSize());
+  Point conditionedParameter;
   if (parameter != currentParameter)
   {
-    std::copy(parameter.begin(), parameter.begin() + conditionedParameter.getSize(), conditionedParameter.begin());
-    distribution_.setParameter(conditionedParameter);
+    if (ResourceMap::GetAsBool("PointConditionalDistribution-UseFullParameters"))
+    {
+      conditionedParameter = Point(distribution_.getParameter().getSize());
+      std::copy(parameter.begin(), parameter.begin() + conditionedParameter.getSize(), conditionedParameter.begin());
+      distribution_.setParameter(conditionedParameter);
+    }
+    // then conditioning values
+    std::copy(parameter.begin() + conditionedParameter.getSize(), parameter.end(), conditioningValues_.begin());
   }
-
-  // then conditioning values
-  std::copy(parameter.begin() + conditionedParameter.getSize(), parameter.end(), conditioningValues_.begin());
   update();
 }
 
 Description PointConditionalDistribution::getParameterDescription() const
 {
-  Description description(distribution_.getParameterDescription());
+  Description description;
+  if (ResourceMap::GetAsBool("PointConditionalDistribution-UseFullParameters"))
+    description.add(distribution_.getParameterDescription());
   description.add(Description::BuildDefault(conditioningIndices_.getSize(), "p_cond_"));
   return description;
 }
