@@ -2,7 +2,7 @@
 /**
  *  @brief Abstract top-level class for all distributions
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -373,7 +373,7 @@ Point Distribution::getRealization() const
   return getImplementation()->getRealization();
 }
 
-/* Get a numerical sample whose elements follow the distribution */
+/* Get a sample whose elements follow the distribution */
 Sample Distribution::getSample(const UnsignedInteger size) const
 {
   return getImplementation()->getSample(size);
@@ -389,6 +389,21 @@ Sample Distribution::getSampleByQMC(const UnsignedInteger size) const
   return getImplementation()->getSampleByQMC(size);
 }
 
+Function Distribution::getPDF() const
+{
+  return getImplementation()->getPDF();
+}
+
+Function Distribution::getLogPDF() const
+{
+  return getImplementation()->getLogPDF();
+}
+
+Function Distribution::getCDF() const
+{
+  return getImplementation()->getCDF();
+}
+
 /* Get the DDF of the distribution */
 Point Distribution::computeDDF(const Point & point) const
 {
@@ -788,29 +803,6 @@ Point Distribution::getSingularities() const
   return getImplementation()->getSingularities();
 }
 
-/* Compute the density generator of the elliptical generator, i.e.
- *  the function phi such that the density of the distribution can
- *  be written as p(x) = phi(t(x-mu)R(x-mu))                      */
-Scalar Distribution::computeDensityGenerator(const Scalar betaSquare) const
-{
-  LOGWARN("Distribution.computeDensityGenerator is deprecated");
-  return getImplementation()->computeDensityGenerator(betaSquare);
-}
-
-/* Compute the derivative of the density generator */
-Scalar Distribution::computeDensityGeneratorDerivative(const Scalar betaSquare) const
-{
-  LOGWARN("Distribution.computeDensityGeneratorDerivative is deprecated");
-  return getImplementation()->computeDensityGeneratorDerivative(betaSquare);
-}
-
-/* Compute the seconde derivative of the density generator */
-Scalar Distribution::computeDensityGeneratorSecondDerivative(const Scalar betaSquare) const
-{
-  LOGWARN("Distribution.computeDensityGeneratorSecondDerivative is deprecated");
-  return getImplementation()->computeDensityGeneratorSecondDerivative(betaSquare);
-}
-
 /* Compute the radial distribution CDF */
 Scalar Distribution::computeRadialDistributionCDF (const Scalar radius,
     const Bool tail) const
