@@ -2,7 +2,7 @@
 /**
  *  @brief Abstract top-level class for elliptical distributions
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2025 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -43,7 +43,7 @@ static const Factory<EllipticalDistribution> Factory_EllipticalDistribution;
 
 /* Default constructor */
 EllipticalDistribution::EllipticalDistribution()
-  : ContinuousDistribution()
+  : DistributionImplementation()
   , sigma_(1, 1.0)
   , R_(1)
 {
@@ -55,7 +55,7 @@ EllipticalDistribution::EllipticalDistribution(const Point & mu,
     const Point & sigma,
     const CorrelationMatrix & R,
     const Scalar covarianceScalingFactor)
-  : ContinuousDistribution()
+  : DistributionImplementation()
   , sigma_(sigma)
   , R_(R)
   , covarianceScalingFactor_(covarianceScalingFactor)
@@ -70,7 +70,7 @@ EllipticalDistribution::EllipticalDistribution(const Point & mu,
         << " mu dimension=" << mu.getDimension();
   // We check that the marginal standard deviations are > 0
   for(UnsignedInteger i = 0; i < dimension; ++i)
-    if (!(sigma[i] > 0.0)) throw InvalidArgumentException(HERE) << "The marginal standard deviations must be > 0 sigma=" << sigma[i];
+    if (!(sigma[i] >= 0.0)) throw InvalidArgumentException(HERE) << "The marginal standard deviations must be >= 0 sigma=" << sigma[i];
   // Then we set the dimension of the Elliptical distribution
   setDimension(dimension);
   // The mean attribute is stored at an upper level
@@ -173,9 +173,10 @@ Scalar EllipticalDistribution::computeDensityGeneratorSecondDerivative(const Sca
 /* Get the DDF of the distribution */
 Point EllipticalDistribution::computeDDF(const Point & point) const
 {
-  if (point.getDimension() != getDimension()) throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=1, here dimension=" << point.getDimension();
-
   const UnsignedInteger dimension = getDimension();
+  if (point.getDimension() != dimension)
+    throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=" << dimension << ", here dimension=" << point.getDimension();
+
   switch(dimension)
   {
     case 1:
@@ -253,9 +254,10 @@ Point EllipticalDistribution::computeDDF(const Point & point) const
 /* Get the PDF of the distribution */
 Scalar EllipticalDistribution::computePDF(const Point & point) const
 {
-  if (point.getDimension() != getDimension()) throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=1, here dimension=" << point.getDimension();
-
   const UnsignedInteger dimension = getDimension();
+  if (point.getDimension() != dimension)
+    throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=" << dimension << ", here dimension=" << point.getDimension();
+
   switch(dimension)
   {
     case 1:
@@ -315,9 +317,10 @@ Scalar EllipticalDistribution::computePDF(const Point & point) const
 /* Get the log-PDF of the distribution */
 Scalar EllipticalDistribution::computeLogPDF(const Point & point) const
 {
-  if (point.getDimension() != getDimension()) throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=1, here dimension=" << point.getDimension();
-
   const UnsignedInteger dimension = getDimension();
+  if (point.getDimension() != dimension)
+    throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=" << dimension << ", here dimension=" << point.getDimension();
+
   switch(dimension)
   {
     case 1:
@@ -386,10 +389,10 @@ Scalar EllipticalDistribution::computeLogPDF(const Point & point) const
 /* Get the PDF gradient of the distribution */
 Point EllipticalDistribution::computePDFGradient(const Point & point) const
 {
-  if (point.getDimension() != getDimension()) throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=1, here dimension=" << point.getDimension();
-
-  const Point minusGardientMean(computeDDF(point));
   const UnsignedInteger dimension = getDimension();
+  if (point.getDimension() != dimension)
+    throw InvalidArgumentException(HERE) << "Error: the given point must have dimension=" << dimension << ", here dimension=" << point.getDimension();
+
   const Point u(normalize(point));
   Point iRu(u);
   for (UnsignedInteger i = 0; i < dimension; ++ i)
@@ -400,14 +403,14 @@ Point EllipticalDistribution::computePDFGradient(const Point & point) const
   const Scalar betaSquare = u.dot(iRu);
   const Scalar phi = computeDensityGenerator(betaSquare);
   const Scalar phiDerivative = computeDensityGeneratorDerivative(betaSquare);
-  Point pdfGradient(2 * dimension + (dimension > 1 ? ((dimension - 1)*dimension) / 2 : 0));
+  Point pdfGradient(2 * dimension + (dimension > 1 ? ((dimension - 1) * dimension) / 2 : 0));
   for (UnsignedInteger i = 0; i < dimension; ++i)
   {
     Scalar iSigma = 1.0 / sigma_[i];
     // dPDF / dmu_i
-    pdfGradient[i] = -2.0 * normalizationFactor_ * phiDerivative * iRu[i] * iSigma;
+    pdfGradient[2 * i] = -2.0 * normalizationFactor_ * phiDerivative * iRu[i] * iSigma;
     // dPDF / dsigma_i
-    pdfGradient[dimension + i] = pdfGradient[i] * u[i] - normalizationFactor_ * phi * iSigma;
+    pdfGradient[2 * i + 1] = pdfGradient[2 * i] * u[i] - normalizationFactor_ * phi * iSigma;
   }
   if (getDimension() > 1)
   {
@@ -481,11 +484,9 @@ void EllipticalDistribution::update()
   const UnsignedInteger dimension = getDimension();
   if (dimension > 1)
   {
-    // Compute the shape matrix
-    const CovarianceMatrix shape(getShape());
     // Try to compute the Cholesky factor of the shape matrix
     TriangularMatrix cholesky(getCholesky());
-    inverseCholesky_ = cholesky.solveLinearSystem(IdentityMatrix(dimension)).getImplementation();
+    inverseCholesky_ = cholesky.inverse().getImplementation();
     normalizationFactor_ = 1.0;
     for (UnsignedInteger i = 0; i < dimension; ++i) normalizationFactor_ /= cholesky(i, i);
   } // dimension > 1
@@ -519,12 +520,6 @@ Point EllipticalDistribution::getMu() const
   return mean_;
 }
 
-void EllipticalDistribution::setMean(const Point & mean)
-{
-  LOGWARN("EllipticalDistribution.setMean is deprecated, use setMu");
-  setMu(mean);
-}
-
 /* Mean computation */
 void EllipticalDistribution::computeMean() const
 {
@@ -590,7 +585,7 @@ void EllipticalDistribution::setR(const CorrelationMatrix & R)
     for (UnsignedInteger j = 0; j <= i; ++j)
       shape(i, j) *= sigma_[i] * sigma_[j];
   TriangularMatrix cholesky(shape.computeRegularizedCholesky());
-  inverseCholesky_ = cholesky.solveLinearSystem(IdentityMatrix(dimension)).getImplementation();
+  inverseCholesky_ = cholesky.inverse().getImplementation();
   R_ = R;
   normalizationFactor_ = 1.0;
   for (UnsignedInteger i = 0; i < dimension; ++ i)
@@ -604,19 +599,6 @@ CorrelationMatrix EllipticalDistribution::getR() const
   return R_;
 }
 
-/** Correlation matrix accessor */
-void EllipticalDistribution::setCorrelation(const CorrelationMatrix & R)
-{
-  LOGWARN("EllipticalDistribution.setCorrelation is deprecated, use setR");
-  setR(R);
-}
-
-CorrelationMatrix EllipticalDistribution::getCorrelation() const
-{
-  LOGWARN("EllipticalDistribution.getCorrelation is deprecated, use getR");
-  return getR();
-}
-
 /* Cholesky factor of the correlation matrix accessor */
 TriangularMatrix EllipticalDistribution::getCholesky() const
 {
@@ -756,7 +738,7 @@ void EllipticalDistribution::setParametersCollection(const PointCollection & par
   setDimension(dimension);
   mean_ = Point(dimension);
   sigma_ = Point(dimension);
-  R_ = CorrelationMatrix(dimension);
+  CorrelationMatrix R(dimension);
   if (dimension == 1)
   {
     mean_[0] = parametersCollection[0][0];
@@ -776,11 +758,11 @@ void EllipticalDistribution::setParametersCollection(const PointCollection & par
     {
       for (UnsignedInteger j = 0; j < i; ++j)
       {
-        R_(i, j) = parametersCollection[size - 1][parameterIndex];
+        R(i, j) = parametersCollection[size - 1][parameterIndex];
         ++parameterIndex;
       }
     }
-    if ( !R_.isPositiveDefinite()) throw InvalidArgumentException(HERE) << "The correlation matrix must be definite positive R=" << R_;
+    setR(R);
   }
   update();
   computeRange();
@@ -836,8 +818,7 @@ void EllipticalDistribution::setParameter(const Point & parameters)
         ++ index;
       }
     }
-    if (!R.isPositiveDefinite())
-      throw InvalidArgumentException(HERE) << "The correlation matrix must be definite positive R=" << R;
+    setR(R);
   }
   // commit all changes at once
   mean_ = mean;
@@ -873,7 +854,7 @@ Description EllipticalDistribution::getParameterDescription() const
 /* Method save() stores the object through the StorageManager */
 void EllipticalDistribution::save(Advocate & adv) const
 {
-  ContinuousDistribution::save(adv);
+  DistributionImplementation::save(adv);
   adv.saveAttribute( "R_", R_ );
   adv.saveAttribute( "sigma_", sigma_ );
   adv.saveAttribute( "mu_", mean_ );
@@ -884,7 +865,7 @@ void EllipticalDistribution::save(Advocate & adv) const
 /* Method load() reloads the object from the StorageManager */
 void EllipticalDistribution::load(Advocate & adv)
 {
-  ContinuousDistribution::load(adv);
+  DistributionImplementation::load(adv);
   adv.loadAttribute( "R_", R_ );
   adv.loadAttribute( "sigma_", sigma_ );
   if (adv.hasAttribute("mean_duplicate"))
