@@ -16,10 +16,8 @@ Define a function with a field output: the viscous free fall example
 # %%
 import openturns as ot
 import openturns.viewer as viewer
-from matplotlib import pylab as plt
 from openturns.usecases import viscous_free_fall
 
-ot.Log.Show(ot.Log.NONE)
 
 # %%
 # Load the viscous free fall example.
@@ -51,14 +49,10 @@ graph.setTitle("Viscous free fall: %d trajectories" % (size))
 graph.setXTitle(r"$t$")
 graph.setYTitle(r"$z$")
 view = viewer.View(graph)
-plt.show()
 
 # %%
 # We see that the object first moves up and then falls down.
 # Not all objects, however, achieve the same maximum altitude.
 # We see that some trajectories reach a higher maximum altitude than others.
 # Moreover, at the final time :math:`t_{max}`, one trajectory hits the ground: :math:`z(t_{max})=0` for this trajectory.
-
-# %%
-# Reset default settings
-ot.ResourceMap.Reload()
+viewer.View.ShowAll()
