@@ -2,6 +2,7 @@
 Use the FORM - SORM algorithms
 ==============================
 """
+
 # %%
 # In this example we estimate a failure probability with the `FORM` algorithm on the :ref:`cantilever beam <use-case-cantilever-beam>` example.
 # More precisely, we show how to use the associated results:
@@ -10,16 +11,10 @@ Use the FORM - SORM algorithms
 # - the probability estimation according to the FORM approximation, and the following SORM ones: Tvedt, Hohenbichler and Breitung,
 # - the Hasofer reliability index and the generalized ones evaluated from the Breitung, Tvedt and Hohenbichler approximations,
 # - the importance factors defined as the normalized director factors of the design point in the :math:`U`-space
-# - the sensitivity factors of the Hasofer reliability index and the FORM probability.
+# - the sensitivity factors of the Hasofer reliability index and the FORM probability,
 # - the coordinates of the mean point in the standard event space.
 #
-# The coordinates of the mean point in the standard event space is:
-#
-# .. math::
-#    \frac{1}{E_1(-\beta)}\int_{\beta}^{\infty} u_1 p_1(u_1)du_1
-#
-#
-# where :math:`E_1` is the spheric univariate distribution of the standard space and :math:`\beta` is the reliability index.
+# See :ref:`FORM <form_approximation>` and :ref:`SORM <sorm_approximation>` for theoretical details.
 
 # %%
 # Model definition
@@ -29,9 +24,8 @@ Use the FORM - SORM algorithms
 from openturns.usecases import cantilever_beam
 import openturns as ot
 import openturns.viewer as viewer
-from matplotlib import pylab as plt
+from matplotlib import pyplot as plt
 
-ot.Log.Show(ot.Log.NONE)
 
 # %%
 # We load the model from the usecases module :
@@ -60,17 +54,18 @@ event.setName("deviation")
 # -------------
 
 # %%
-# Define a solver
-optimAlgo = ot.Cobyla()
+# Define a solver, here we use a :class:`~openturns.MultiStart` optimization based on :class:`~openturns.Cobyla`
+startingSample = distribution.getSample(10)
+optimAlgo = ot.MultiStart(ot.Cobyla(), startingSample)
 optimAlgo.setMaximumCallsNumber(1000)
-optimAlgo.setMaximumAbsoluteError(1.0e-10)
-optimAlgo.setMaximumRelativeError(1.0e-10)
-optimAlgo.setMaximumResidualError(1.0e-10)
-optimAlgo.setMaximumConstraintError(1.0e-10)
+optimAlgo.setMaximumAbsoluteError(1.0e-4)
+optimAlgo.setMaximumRelativeError(1.0e-4)
+optimAlgo.setMaximumResidualError(1.0e-4)
+optimAlgo.setMaximumConstraintError(1.0e-4)
 
 # %%
 # Run FORM
-algo = ot.FORM(optimAlgo, event, distribution.getMean())
+algo = ot.FORM(optimAlgo, event)
 algo.run()
 result = algo.getResult()
 
@@ -125,7 +120,7 @@ view = viewer.View(graphErrors)
 
 # %%
 # Get additional results with SORM
-algo = ot.SORM(optimAlgo, event, distribution.getMean())
+algo = ot.SORM(optimAlgo, event)
 algo.run()
 sorm_result = algo.getResult()
 
@@ -195,6 +190,6 @@ event.setName("deviation")
 
 # %%
 # We can then run the FORM analysis in the same way as before:
-algo = ot.FORM(optimAlgo, event, distribution.getMean())
+algo = ot.FORM(optimAlgo, event)
 algo.run()
 result = algo.getResult()
