@@ -12,6 +12,7 @@ Examples
 >>> view.save('graph.png', dpi=100)
 >>> view.show()
 """
+
 import openturns as ot
 import numpy as np
 import matplotlib
@@ -21,9 +22,9 @@ import warnings
 import io
 
 try:
-    from pkg_resources import parse_version
+    from packaging.version import Version
 except ImportError:
-    from packaging.version import Version as parse_version
+    from pkg_resources import parse_version as Version
 
 __all__ = ["View", "PlotDesign"]
 
@@ -92,7 +93,6 @@ class RankNormalize(cls.Normalize):
 
 
 class View:
-
     """
     Create the figure.
 
@@ -159,6 +159,9 @@ class View:
     legend_kw : dict, optional
         Passed on to matplotlib.axes.Axes.legend kwargs
 
+    colorbar_kw : dict, optional
+        Passed on to matplotlib.figure.Figure.colorbar kwargs
+
     add_legend : bool, optional
         Adds a legend if True. Default is True.
 
@@ -211,9 +214,10 @@ class View:
         scatter_kw=None,
         text_kw=None,
         legend_kw=None,
+        colorbar_kw=None,
         add_legend=True,
         square_axes=False,
-        **kwargs
+        **kwargs,
     ):
         if not (
             isinstance(graph, ot.Graph)
@@ -233,7 +237,7 @@ class View:
                 )
 
         # Store matplotlib version
-        matplotlib_version = parse_version(matplotlib.__version__)
+        matplotlib_version = Version(matplotlib.__version__)
 
         # check that arguments are dictionaries
         figure_kw = self._CheckDict(figure_kw)
@@ -249,6 +253,7 @@ class View:
         scatter_kw_default = self._CheckDict(scatter_kw)
         text_kw_default = self._CheckDict(text_kw)
         legend_kw_default = self._CheckDict(legend_kw)
+        colorbar_kw_default = self._CheckDict(colorbar_kw)
         legend_handles = []
         legend_labels = []
 
@@ -319,6 +324,7 @@ class View:
                         contour_kw=contour_kw,
                         step_kw=step_kw,
                         clabel_kw=clabel_kw,
+                        scatter_kw=scatter_kw,
                         text_kw=text_kw,
                         legend_kw=legend_kw,
                     )
@@ -417,7 +423,7 @@ class View:
             polygoncollection_kw["zorder"] = zorder
             contour_kw["zorder"] = zorder
             step_kw["zorder"] = zorder
-            if matplotlib_version >= parse_version("3.3"):
+            if matplotlib_version >= Version("3.3"):
                 clabel_kw["zorder"] = zorder
             scatter_kw["zorder"] = zorder
             text_kw["zorder"] = zorder
@@ -574,14 +580,14 @@ class View:
                     "facecolors" not in polygoncollection_kw_default
                     and "fc" not in polygoncollection_kw_default
                 ):
-                    polygoncollection_kw[
-                        "facecolors"
-                    ] = drawable.getPaletteAsNormalizedRGBA()
+                    polygoncollection_kw["facecolors"] = (
+                        drawable.getPaletteAsNormalizedRGBA()
+                    )
 
                 self._ax[0].add_collection(
                     matplotlib.collections.PolyCollection(
                         np.array(data).reshape((polygonsNumber, verticesNumber, 2)),
-                        **polygoncollection_kw
+                        **polygoncollection_kw,
                     )
                 )
 
@@ -596,9 +602,12 @@ class View:
             elif drawableKind == "Contour":
                 contour = drawable.getImplementation()
                 X, Y = np.meshgrid(drawable.getX(), drawable.getY())
+                # X values move with columns
+                # Y values move with rows
+                # Z shape is therefore (getY().getSize(), getX().getSize())
                 Z = np.reshape(
                     drawable.getData(),
-                    (drawable.getX().getSize(), drawable.getY().getSize()),
+                    (drawable.getY().getSize(), drawable.getX().getSize()),
                 )
                 if len(drawable.getLevels()) > 0:
                     contour_kw.setdefault("levels", drawable.getLevels())
@@ -647,20 +656,28 @@ class View:
                     if "norm" in contour_kw_default
                     else contour.getColorMapNorm()
                 )
-                if type(norm) is str and matplotlib_version < parse_version("3.6.0"):
+                if type(norm) is str and matplotlib_version < Version("3.6.0"):
                     # matplotlib before 3.6 does not support norms as strings
                     try:
                         normDict = {
                             "rank": "rank",
                             "linear": cls.Normalize(),
                             "log": cls.LogNorm(),
-                            "symlog": cls.SymLogNorm(linthresh=0.03)
-                            if matplotlib_version < parse_version("3.2.0")
-                            else cls.SymLogNorm(linthresh=0.03, base=10),
+                            "symlog": (
+                                cls.SymLogNorm(linthresh=0.03)
+                                if matplotlib_version < Version("3.2.0")
+                                else cls.SymLogNorm(linthresh=0.03, base=10)
+                            ),
                         }
                         contour_kw["norm"] = normDict[norm]
                     except KeyError:
                         warnings.warn("-- Unknown norm " + norm)
+
+                    # 3.3.4 (debian 11) does not like null log levels
+                    if norm == "log" and "levels" in contour_kw:
+                        contour_kw["levels"] = list(
+                            filter(lambda x: x > 0.0, contour_kw["levels"])
+                        )
                 else:
                     contour_kw["norm"] = norm
                 if contour_kw.get("norm") == "rank":
@@ -678,7 +695,23 @@ class View:
                 else:
                     contourset = self._ax[0].contour(X, Y, Z, **contour_kw)
                 self._contoursets.append(contourset)
-                if drawable.getDrawLabels() and not contour.isFilled():
+
+                # matplotlib may fail with ValueError exception "RGBA sequence should have length 3 or 4'"
+                # can either disable labels or color map to work around it
+                draw_labels = drawable.getDrawLabels()
+                if (
+                    draw_labels
+                    and len(drawable.getLevels()) == 1
+                    and contour_kw.get("cmap", "") != ""
+                    and matplotlib_version >= Version("3.6")
+                    and matplotlib_version < Version("3.7")
+                ):
+                    warnings.warn(
+                        "openturns.viewer: disabling contour labels to work around matplotlib 3.6.x issue"
+                    )
+                    draw_labels = False
+
+                if draw_labels and not contour.isFilled():
                     # Matplotlib does not support labels in filled contours well
                     clabel_kw.setdefault("fontsize", 8)
                     # Use labels
@@ -703,23 +736,29 @@ class View:
                     legend_labels.append(drawable.getLegend())
                 if contour.getColorBarPosition() and len(contour.getLevels()) != 1:
                     colorbar = None
-                    if matplotlib_version >= parse_version("3.7.0"):
-                        colorbar = self._fig.colorbar(
-                            contourset,
-                            location=contour.getColorBarPosition(),
-                            format="%.3g",
-                        )
+                    colorbar_kw = dict(colorbar_kw_default)
+                    if "format" not in colorbar_kw:
+                        colorbar_kw["format"] = "%.3g"
+                    if matplotlib_version >= Version("3.7.0"):
+                        if "location" not in colorbar_kw:
+                            colorbar_kw["location"] = contour.getColorBarPosition()
+                        colorbar = self._fig.colorbar(contourset, **colorbar_kw)
                     else:
-                        try:
-                            colorbar = self._fig.colorbar(contourset, format="%.3g")
-                            if contour.getColorBarPosition() != "right":
+                        # do not draw colorbar when Z range has too few values
+                        # https://github.com/matplotlib/matplotlib/issues/23817
+                        nZ = len(set([z[0] for z in drawable.getData()]))
+                        if nZ > 2:
+                            try:
+                                colorbar = self._fig.colorbar(contourset, **colorbar_kw)
+                                if contour.getColorBarPosition() != "right":
+                                    warnings.warn(
+                                        "-- colorbar location was not used in matplotlib < 3.7.0"
+                                    )
+                            except ZeroDivisionError:
                                 warnings.warn(
-                                    "-- colorbar location was not used in matplotlib < 3.7.0"
+                                    "figure.colorbar likely failed on boundary levels"
                                 )
-                        except ZeroDivisionError:
-                            warnings.warn(
-                                "figure.colorbar likely failed on boundary levels"
-                            )
+
                     if colorbar is not None and contour.getLevels():
                         colorbar.formatter.minor_thresholds = (
                             10.0,
@@ -781,7 +820,7 @@ class View:
                         text,
                         horizontalalignment=horizontal,
                         verticalalignment=vertical,
-                        **text_kw
+                        **text_kw,
                     )
 
             else:
@@ -823,7 +862,7 @@ class View:
             legend_kw.setdefault("shadow", True)
 
             # by default legend is a bit too large
-            legend_kw.setdefault("prop", {"size": 10})
+            legend_kw.setdefault("fontsize", int(graph.getLegendFontSize()))
 
             if len(legend_handles) > 0:
                 legend = self._ax[0].legend(legend_handles, legend_labels, **legend_kw)
@@ -1116,8 +1155,7 @@ def PlotDesign(
         bounds = ot.Interval(lowerBound, upperBound)
     if bounds.getDimension() != dim:
         raise ValueError(
-            "Dimension of bounds %d do not match the dimension of the sample %d"
-            % (bounds.getDimension(), dim)
+            f"Dimension of bounds ({bounds.getDimension}) do not match the dimension of the sample ({dim})"
         )
 
     # Check the subdivisions
@@ -1126,8 +1164,7 @@ def PlotDesign(
         subdivisions = [size] * dim
     if len(subdivisions) != dim:
         raise ValueError(
-            "Number of subdivisions %d does not match the dimension of the sample %d"
-            % (len(subdivisions), dim)
+            f"Number of subdivisions ({len(subdivisions)}) does not match the dimension of the sample ({dim})"
         )
 
     # Get description
