@@ -68,9 +68,7 @@ formula[1] = (
 covTh_Sobol -= 1
 # Reference analytical values
 meanTh_Ishigami = a / 2.0
-covTh_Ishigami = (
-    b**2 * m.pi**8 / 18.0 + (b * m.pi**4) / 5.0 + a**2 / 8.0 + 1.0 / 2.0
-)
+covTh_Ishigami = b**2 * m.pi**8 / 18.0 + (b * m.pi**4) / 5.0 + a**2 / 8.0 + 1.0 / 2.0
 sob_1_Ishigami = ot.Point(3)
 sob_1_Ishigami[0] = (
     b * m.pi**4 / 5.0 + b**2 * m.pi**8 / 50.0 + 1.0 / 2.0
@@ -79,9 +77,7 @@ sob_1_Ishigami[1] = (a**2 / 8.0) / covTh_Ishigami
 sob_1_Ishigami[2] = 0.0
 sob_2_Ishigami = ot.Point(3)
 sob_2_Ishigami[0] = 0.0
-sob_2_Ishigami[1] = (
-    b**2 * m.pi**8 / 18.0 - b**2 * m.pi**8 / 50.0
-) / covTh_Ishigami
+sob_2_Ishigami[1] = (b**2 * m.pi**8 / 18.0 - b**2 * m.pi**8 / 50.0) / covTh_Ishigami
 sob_2_Ishigami[2] = 0.0
 sob_3_Ishigami = ot.Point(1, 0.0)
 # Multidimensional reference values
@@ -193,10 +189,6 @@ for adaptiveStrategyIndex in range(len(listAdaptiveStrategy)):
         print("###################################")
         print(algo.getAdaptiveStrategy())
         print(algo.getProjectionStrategy())
-        residuals = result.getResiduals()
-        print("residuals=", residuals)
-        relativeErrors = result.getRelativeErrors()
-        print("relative errors=", relativeErrors)
 
         # Post-process the results
         vector = ot.FunctionalChaosRandomVector(result)
